import yaml
import json


def load_and_merge(file_path=None, defaults=None):
    """
    Load a given file and apply defaults as specified to any values not present in the file.
    Does a merge of the file and defaults and returns a dict with the results
    :param file_path: file to load. Supports .yaml and .json files
    :param defaults: dict with defaults in correct structure to compare and overlay with the values from the file
    :return: a new dict with the merged results
    """

    ret = None
    if defaults is not None:
        ret = defaults.copy()

    file_data = None
    if file_path is not None:
        with open(file_path) as f:
            if file_path.endswith('.yaml') or file_path.endswith('.yml'):
                file_data = yaml.safe_load(f)
            elif file_path.endswith('.json'):
                file_data = json.load(f.read())
    else:
        file_data = {}

    if file_data is not None:
        if ret is not None:
            ret.update(file_data)
            return ret
        else:
            return file_data
    else:
        return ret



import json

import os
import time
import anchore_utils

DEVNULL = open(os.devnull, 'wb')


class AnchoreImageDB(object):
    _db_metadata_filename = 'anchore_db_meta.json'

    def __del__(self):
        if self.initialized:
            pass

    @staticmethod
    def exists(path, with_expected_db_version=None):
        """
        Does the db exist, optionally with the expected version
        :param path:
        :param with_expected_db_version:
        :return:
        """
        dbmetafile = os.path.join(path, AnchoreImageDB._db_metadata_filename)

        if not os.path.exists(path) or not os.path.exists(dbmetafile):
            return False

        if with_expected_db_version:
            with open(dbmetafile, 'r') as f:
                json_dict = json.load(f)

            if json_dict['anchore_db_version'] != with_expected_db_version:
                return False
        else:
            return True

    def __init__(self, imagerootdir):
        self.initialized = False
        self.imagerootdir = imagerootdir

        try:
            from anchore import version as anchore_version_string
            if not os.path.exists(self.imagerootdir):
                raise Exception("anchore DB: image root dir does not exist "+str(imagerootdir))

            # check that the software and DB are same version before continuing
            dbmetafile = '/'.join([self.imagerootdir, "anchore_db_meta.json"])
            if not os.path.exists(dbmetafile):
                FH=open(dbmetafile, 'w')
                FH.write(json.dumps({}))
                FH.close()

            update = False

            FH=open(dbmetafile, 'r')
            json_dict = json.loads(FH.read())
            FH.close()
            if 'anchore_version' not in json_dict:
                json_dict['anchore_version'] = anchore_version_string
                update = True
            if 'anchore_db_version' not in json_dict:
                json_dict['anchore_db_version'] = anchore_version_string
                update = True
            elif json_dict['anchore_db_version'] < anchore_version_string:
                # Check major version
                local_db_version = json_dict['anchore_db_version']
                db_version_tuple = tuple(local_db_version.split('.'))
                code_version_tuple = tuple(anchore_version_string.split('.'))
                if db_version_tuple[0] < code_version_tuple[0]:
                    raise Exception("Anchore DB version ("+local_db_version + ") " +
                                    "is at least one major-version behind the Anchore software version (" + anchore_version_string + ")")

            if update:
                FH=open(dbmetafile, 'w')
                FH.write(json.dumps(json_dict))
                FH.close()

        except Exception as err:
            raise err

        self.initialized = True

    def load_all_images(self):
        ret = []
        
        for d in os.listdir(self.imagerootdir):
            try:
                result = self.load_image(d)
                if result['meta']['imageId'] not in ret:
                    ret.append(result['meta']['imageId'])
            except Exception as err:
                pass
            
        return(ret)

    def load_image(self, imageId):
        # populate all anchore data into image
        imagedir = self.imagerootdir + "/" + imageId
        analyzer_meta = {}
        allfiles = {}
        allpkgs = {}
        familytree = []
        layers = []

        if not os.path.exists(imagedir):
            # image has not been analyzed
            return(False)

        report = self.load_analysis_report(imageId)

        if 'file_checksums' in report:
            filelist = report['file_checksums'].pop('files.md5sums', None)
            for line in filelist:
                line = line.strip()
                (k, v) = line.split()
                allfiles[k] = v

            
        if 'package_list' in report:
            pkglist = report['package_list'].pop('pkgs.all', None)
            for line in pkglist:
                line = line.strip()
                (k, v) = line.split()
                allpkgs[k] = v

        if 'analyzer_meta' in report:
            ameta = report['analyzer_meta'].pop('analyzer_meta', None)
            analyzer_meta = {}
            for line in ameta:
                line = line.strip()
                (key, val) = line.split()
                analyzer_meta[key] = val

        report = self.load_image_report(imageId)

        familytree = report.pop('familytree', list())        
        layers = report.pop('layers', list())
        meta = report.pop('meta', {})
        all_tags = report.pop('anchore_all_tags', list())
        current_tags = report.pop('anchore_current_tags', list())
        tag_history = report.pop('tag_history', list())

        ret = {}
        ret['meta'] = meta
        ret['allfiles'] = allfiles
        ret['allpkgs'] = allpkgs
        ret['familytree'] = familytree
        ret['layers'] = layers
        ret['analyzer_meta'] = analyzer_meta
        ret['all_tags'] = all_tags
        ret['current_tags'] = current_tags
        ret['tag_history'] = tag_history
        
        return(ret)

    def load_analysis_report(self, imageId):
        thefile = self.imagerootdir + "/" + imageId + "/reports/analysis_report.json"
        if not os.path.exists(thefile):
            return({})

        FH = open(thefile, 'r')
        ret = json.loads(FH.read())
        FH.close()
        return(ret)

    def save_analysis_report(self, imageId, report):
        thedir = self.imagerootdir + "/" + imageId + "/reports/"
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = thedir + "/analysis_report.json"
        anchore_utils.update_file_jsonstr(json.dumps(report), thefile, False)

    def load_compare_report(self, imageId):
        thefile = self.imagerootdir + "/" + imageId + "/reports/compare_report.json"
        if not os.path.exists(thefile):
            return({})

        FH = open(thefile, 'r')
        ret = json.loads(FH.read())
        FH.close()
        
        return(ret)

    def save_compare_report(self, imageId, report):
        thedir = self.imagerootdir + "/" + imageId + "/reports/"
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = thedir + "/compare_report.json"
        anchore_utils.update_file_jsonstr(json.dumps(report), thefile, False)

    def load_gates_report(self, imageId):
        thefile = self.imagerootdir + "/" + imageId + "/reports/gates_report.json"
        if not os.path.exists(thefile):
            return({})

        FH = open(thefile, 'r')
        ret = json.loads(FH.read())
        FH.close()
        return(ret)

    def save_gates_report(self, imageId, report):
        thedir = self.imagerootdir + "/" + imageId + "/reports/"
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = thedir + "/gates_report.json"
        anchore_utils.update_file_jsonstr(json.dumps(report), thefile, False)

    def load_gates_eval_report(self, imageId):
        thefile = self.imagerootdir + "/" + imageId + "/reports/gates_eval_report.json"
        if not os.path.exists(thefile):
            return({})

        FH = open(thefile, 'r')
        ret = json.loads(FH.read())
        FH.close()
        return(ret)

    def save_gates_eval_report(self, imageId, report):
        thedir = self.imagerootdir + "/" + imageId + "/reports/"
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = thedir + "/gates_eval_report.json"
        anchore_utils.update_file_jsonstr(json.dumps(report), thefile, False)

    def load_image_report(self, imageId):
        thefile = self.imagerootdir + "/" + imageId + "/reports/image_report.json"
        if not os.path.exists(thefile):
            return({})

        FH = open(thefile, 'r')
        ret = json.loads(FH.read())
        FH.close()
        return(ret)

    def save_image_report(self, imageId, report):
        date = str(int(time.time()))
        thedir = self.imagerootdir + "/" + imageId + "/reports/"
        if not os.path.exists(thedir):
            os.makedirs(thedir)
        thefile = thedir + "/image_report.json"

        if os.path.exists(thefile):
            oldreport = self.load_image_report(imageId)
            if 'tag_history' in oldreport:
                report['tag_history'] = list(oldreport['tag_history'])
            else:
                report['tag_history'] = list()

            if 'anchore_current_tags' not in oldreport:
                oldreport['anchore_current_tags'] = list([date, report['anchore_current_tags']])

            diff = list(set(oldreport['anchore_current_tags']).symmetric_difference(set(report['anchore_current_tags'])))
            if len(diff) > 0:
                # there is a difference between stored tags and new tags
                report['tag_history'].append([date, oldreport['anchore_current_tags']])
        
        if 'tag_history' not in report:
            report['tag_history'] = list()

        if len(report['tag_history']) <= 0:
            report['tag_history'].append([date, report['anchore_current_tags']])

        anchore_utils.update_file_jsonstr(json.dumps(report), thefile, False)



import os
import json
import datetime
import hashlib

import requests
import logging
import shutil
import subprocess

from clint.textui import progress

import fs_util
from anchore.util import RFC1123_TIME_FORMAT


class ResourceRetriever(object):
    """
    Interface for fetching remote resources

    """

    def __init__(self, resource_url):
        self.url = resource_url

    def fetch(self, output_path):
        raise NotImplementedError('Must be implemented by subclass')

    def get_metadata(self):
        raise NotImplementedError('Must be implemented by subclass')


user_agent_string = None


def get_user_agent():
    """
    Construct and informative user-agent string.
    Includes OS version and docker version info

    Not thread-safe
    :return:
    """

    global user_agent_string
    if user_agent_string is None:
        # get OS type
        try:
            sysinfo = subprocess.check_output('lsb_release -i -r -s'.split(' '))
            dockerinfo = subprocess.check_output('docker --version'.split(' '))

            user_agent_string = ' '.join([sysinfo.replace('\t','/'),
                                          dockerinfo.replace(" version ", "/").replace(", ", "/").replace(' ', '/')]).replace('\n','')
        except:
            user_agent_string = None

    return user_agent_string


class HttpResourceRetriever(ResourceRetriever):
    _logger = logging.getLogger(__name__)

    chunk_size = 1024 * 1024 # Bytes for each bar of the progress bar

    def fetch(self, output_path, last_etag=None, lastmodified=None, progress_bar=False):
        """
        :param output_path:
        :param last_etag:
        :param lastmodified:
        :param progress_bar:
        :return:
        """
        req_headers = {}
        usr_agent = get_user_agent()
        if usr_agent is not None:
            req_headers['User-Agent'] = usr_agent

        if last_etag is not None:
            req_headers['If-None-Match'] = '"' + last_etag + '"'
        if lastmodified is not None:
            req_headers['If-Modified-Since'] = lastmodified

        response = requests.get(self.url, headers=req_headers, stream=progress_bar)

        if response.status_code == 304:
            # Conditions didn't match, no content returned
            return None
        if response.status_code != 200:
            raise requests.HTTPError

        with open(output_path, 'wb') as f:
            if progress_bar:
                total_size = long(response.headers.get('content-length'))
                for chunk in progress.bar(response.iter_content(chunk_size=self.chunk_size),
                                          expected_size=(total_size / self.chunk_size) + 1):
                    if chunk:
                        f.write(chunk)
                        f.flush()
            else:
                f.write(response.content)

        headers = normalize_headers(response.headers)

        if 'etag' not in headers:
            headers['etag'] = fs_util.calc_file_md5(output_path)

        return headers

    def get_metadata(self):
        response = requests.head(self.url)
        if response.status_code != 200:
            return None
        else:
            return normalize_headers(response.headers)


def normalize_headers(headers):
    """
    Convert useful headers to a normalized type and return in a new dict

    Only processes content-type, content-length, etag, and last-modified
    :param headers:
    :return:
    """
    for (k, v) in headers.items():
        headers.pop(k)
        headers[k.lower()] = v

    result = {}
    if 'content-length' in headers:
        result['content-length'] = headers['content-length']
    elif 'contentlength' in headers:
        result['content-length'] = headers['contentlength']

    if 'content-type' in headers:
        result['content-type'] = headers['content-type']
    elif 'contenttype' in headers:
        result['content-type'] = headers['contenttype']

    if 'last-modified' in headers:
        result['last-modified'] = headers['last-modified']
    elif 'lastmodified' in headers:
        result['last-modified'] = headers['lastmodified']
    else:
        result['last-modified'] = datetime.datetime.utcnow().strftime(RFC1123_TIME_FORMAT)

    # Normalize date
    if isinstance(result['last-modified'], datetime.datetime):
        result['last-modified'] = result['last-modified'].strftime(RFC1123_TIME_FORMAT)

    if 'ETag' in headers:
        result['etag'] = headers['etag'].strip('"')
    elif 'etag' in headers:
        result['etag'] = headers['etag'].strip('"')

    return result

def get_resource_retriever(url):
    """
    Get the appropriate retriever object for the specified url based on url scheme.
    Makes assumption that HTTP urls do not require any special authorization.

    For HTTP urls: returns HTTPResourceRetriever
    For s3:// urls returns S3ResourceRetriever

    :param url: url of the resource to be retrieved
    :return: ResourceRetriever object
    """

    if url.startswith('http://') or url.startswith('https://'):
        return HttpResourceRetriever(url)
    else:
        raise ValueError('Unsupported scheme in url: %s' % url)


class ResourceCache(object):
    """
    A Resource cache backed by a local directory.
    Composed of a storage directory with content files (named by etag) and a metadata file
    that preserves HTTP headers and maps the content files to the URLs from whence they came. Handles the case of
    the same content via different URLs (common with S3, for example).
    """
    _logger = logging.getLogger(__name__)
    metadata_filename = 'cache_metadata.json'

    def __init__(self, path='/tmp', progress_bars=False):
        self.path = path
        self.progress_bars = progress_bars
        #self.progress_bars = False

        self.metadata_file = os.path.join(self.path, self.metadata_filename)
        self.metadata = {}

        if not os.path.exists(self.metadata_file):
            self._logger.debug('Initializing resource cache at %s' % self.path)
            fs_util.createpath(path=self.path, mode=0744, exists_ok=True)
            with open(self.metadata_file, 'w') as f:
                json.dump(self.metadata, f)
        else:
            with open(self.metadata_file) as f:
                self.metadata = json.load(f)

        self._validate_cache()

    def _validate_cache(self):
        for url, entry in self.metadata.items():
            # Remove any where file is not found. Can occur due to use Ctrl-C or unexpected failure
            if not os.path.exists(entry['content']):
                self.metadata.pop(url)

    def lookup(self, url):
        """
        Like 'get', but only returns metadata if present. Will *not* download/load the cache
        :param url:
        :return:
        """
        return self.metadata[url]

    def check(self, url):
        return url in self.metadata

    def is_stale(self, url):
        self._logger.debug('Checking staleness of url %s into resource cache' % url)
        retriever = get_resource_retriever(url)

        try:
            if url in self.metadata:
                headers = retriever.get_metadata()
                if 'etag' in headers:
                    return self.metadata[url]['etag'] != headers['etag']
                elif 'lastmodified' in headers:
                    return self.metadata[url]['lastmodified'] != headers['lastmodified']

            else:
                return True
        except Exception, e:
            self._logger.debug('Error checking staleness of %s: %s' % (url, e.message))
            raise

    def get(self, url):
        """
        Lookup the given url and return an entry if found. Else return None.
        The returned entry is a dict with metadata about the content and a 'content' key with a file path value

        :param url:
        :return:
        """
        self._load(url)
        self._flush()

        return self.metadata[url]

    def is_empty(self):
        return len(self.metadata) == 0

    def clear(self):
        """
        Remove all entries from the cache and delete all data.
        :return:
        """
        for f in [x['content'] for x in self.metadata.values()]:
            os.remove(f)

        self.metadata = {}
        self._flush()

    @staticmethod
    def _hash_path(url):
        return hashlib.md5(url).hexdigest()

    def _load(self, url):
        """
        Load from remote, but check local file content to identify duplicate content. If local file is found with
        same hash then it is used with metadata from remote object to avoid fetching full content.

        :param url:
        :return:
        """
        self._logger.debug('Loading url %s into resource cache' % url)
        retriever = get_resource_retriever(url)
        content_path = os.path.join(self.path, self._hash_path(url))

        try:
            if url in self.metadata:
                headers = retriever.fetch(content_path, last_etag=self.metadata[url]['etag'], progress_bar=self.progress_bars)
                if headers is None:
                    # no update, return what is already loaded
                    self._logger.info('Cached %s is up-to-date. No data download needed' % url)
                    return self.metadata[url]
            else:
                headers = retriever.fetch(content_path, progress_bar=self.progress_bars)
                if headers is None:
                    raise Exception('Fetch of %s failed' % url)

            if 'etag' not in headers:
                # No Etag returned, so generate it
                headers['etag'] = fs_util.calc_file_md5(content_path)

            # Populate metadata from the headers
            self.metadata[url] = headers.copy()
            self.metadata[url]['content'] = content_path
            return self.metadata[url]
        except:
            self._logger.error('Failed getting resource: %s')

            # forcibly flush local entry if found
            if url in self.metadata:
                self.metadata.pop(url)
            raise
        finally:
            if url not in self.metadata:
                self._logger.debug('Cleaning up on failed load of %s' % url)
                # Cleanup on failure
                if content_path is not None and os.path.exists(content_path):
                    os.remove(content_path)

    def _flush(self):
        """
        Flush metadata to the backing file
        :return:
        """
        with open(self.metadata_file, 'w') as f:
            json.dump(self.metadata, f)

    #def __del__(self):
        # Ensure the lock is released
        #fcntl.flock(self._metadata_fd, fcntl.LOCK_UN)



# Main entry point for the registry manager cli
import click

from anchore.catalog import AnchoreCatalog
from anchore.cli.common import anchore_print, anchore_print_err

working_catalog = None


@click.group()
@click.pass_obj
def subscriptions(anchore_config):
    """
    Manage local subscriptions. Subscriptions are selections from the anchore service for images to pull. These commands
    affect the set of images that will get pulled down during a catalog sync. see anchore-sync-catalog(1).

    The list of available tags must be populated from the anchore service, so you first must run 'anchore sync catalog'
    to get populate the list of options.

    Examples:
    \b
    anchore subscriptions show --tags ubuntu
    anchore subscriptions add ubuntu:wiley

    """
    assert anchore_config is not None

    # Initialize the registry object
    global working_catalog
    try:
        working_catalog = AnchoreCatalog(config=anchore_config)
    except:
        anchore_print_err('Failed loading catalog. Please check config file: %s' % anchore_config.config_file)
        exit(2)


@subscriptions.command(short_help='Show available subscription options.')
@click.option('--subscribed', is_flag=True, default=False, help='Show only the current subscription values')
@click.option('--tags', is_flag=True, default=False, help='Show available tags, not just repo names')
@click.argument('filters', nargs=-1, default=None, metavar='<repo>')
def show(subscribed, tags, filters):
    """
    Show the subscription options available and/or the current subscription list.

    Examples:

    Show all tags for all available repos for subscription: 'anchore subscriptions show --tags'
    Show all tags for centos available for subscription: 'anchore subscriptions show centos --tags'
    Show only the current subscriptions: 'anchore subscriptions show --subscribed'

    """

    if not working_catalog.has_db():
        anchore_print_err('No local analysis db detected. You probably need to run "anchore sync catalog" first to initialize')
        exit(5)

    results = {}
    try:
        if filters:
            results['Current Subscription'] = filter(lambda x: x in filters, working_catalog.subscription.get())
        else:
            results['Current Subscription'] = working_catalog.subscription.get()

    except:
        anchore_print_err('Failed getting subscription data.')
        exit(1)

    if not subscribed:
        repos = working_catalog.metadata.engine_repos
        tag_set = working_catalog.metadata.engine_tags
        if filters:
            filtered_repos = filter(lambda x: (x in repos) or (x in tag_set), filters)
        else:
            filtered_repos = repos

        if not tags:
            results['Available'] = filtered_repos
        else:
            results['Available'] = {}
            for r in filtered_repos:
                results['Available'][r] = filter(lambda x: x.startswith(r), tag_set)

    anchore_print(results, do_formatting=True)
    return


@subscriptions.command(short_help='Add image tags to the subscription list')
@click.argument('repos', nargs=-1, metavar='<repo or tag>')
def add(repos):
    """
    Adds the specified images/tags to the subscription. Tags are formatted as docker image tags. Values are checked
    against the list of available tags from the service. Run 'anchore subscriptions show' to get the list of available
    options. Because of this, you must run an initial 'anchore sync catalog' before subscription data is available.

    Duplicate entries are prevented and will not result in an error, but will be discarded.

    Tag/repo examples: ubuntu, centos:7, nginx:latest

    """

    if not working_catalog.has_db():
        anchore_print_err('No local analysis db detected. You probably need to run "anchore sync catalog" first to initialize')
        exit(5)

    repo_list = list(repos)

    try:
        working_catalog.subscribe(repo_list)
        if working_catalog.configuration().cliargs['json']:
            anchore_print(working_catalog.subscription.get(), do_formatting=True)
        else:
            anchore_print('\n'.join(working_catalog.subscription.get()))
    except:
        anchore_print_err('Failed adding %s to subscription' % repo_list)
        exit(1)


@subscriptions.command(name='rm', short_help='Remove entries from the subscription list')
@click.argument('repos', nargs=-1, metavar='<repo or tag>')
def remove_subscription(repos):
    """
    Removes the specified images/tags to the subscription. Tags are formatted as docker image tags. Accepts values from
    the list of current subscriptions, reachable with 'anchore subscriptions show --subscribed'.
    """

    if not working_catalog.has_db():
        anchore_print_err('No local analysis db detected. You probably need to run "anchore sync catalog" first to initialize')
        exit(5)

    repo_list = list(repos)
    try:
        working_catalog.unsubscribe(repo_list)
        if working_catalog.configuration().cliargs['json']:
            anchore_print(working_catalog.subscription.get(), do_formatting=True)
        else:
            anchore_print('\n'.join(working_catalog.subscription.get()))

    except:
        anchore_print_err('Failed adding %s to subscription' % repo_list)
        exit(1)







import yaml
import os
import shutil
from util.tools import load_and_merge
from pkg_resources import Requirement, resource_filename


class AnchoreConfiguration (object):
    """
    Backed by a yaml file
    """

    DEFAULT_ANCHORE_DATA_DIR = os.path.join(os.getenv('HOME'), '.anchore')
    DEFAULT_CONFIG_DIR = os.path.join(DEFAULT_ANCHORE_DATA_DIR, 'conf')
    DEFAULT_CONFIG_FILE = os.path.join(DEFAULT_CONFIG_DIR, 'config.yaml')
    EXAMPLE_CONFIG_DIR = resource_filename("anchore", "conf/")
    EXAMPLE_CONFIG_FILE = resource_filename("anchore", "conf/config.yaml")
    DEFAULT_PKG_DIR = resource_filename("anchore", "")
    DEFAULT_SCRIPTS_DIR = resource_filename("anchore", "anchore-modules")

    DEFAULTS = {
        'anchore_data_dir': DEFAULT_ANCHORE_DATA_DIR,
        'image_data_store': 'data',
        'tmpdir': '/tmp',
        'pkg_dir': DEFAULT_PKG_DIR,
        'scripts_dir': DEFAULT_SCRIPTS_DIR,
        'docker_conn': 'unix://var/run/docker.sock',
        'vulnerabilities': {
            'url': 'https://service-data.anchore.com/vulnerabilities.tar.gz',
        },
        'analysis': {
            'url': 'https://service-data.anchore.com/analysis_db.tar.gz',
        },
        'images': {
            'docker_conn': 'unix://var/run/docker.sock'
        }
    }

    def __init__(self, cliargs=None):
        # config file handling
        self.config_dir, self.config_file = self.find_config_file()
        try:
            self.data = load_and_merge(file_path=self.config_file, defaults=self.DEFAULTS)

            # set up names of CLI common contexts
            #self.contexts = {}
            #self.contexts['docker_cli'] = None
            #self.contexts['anchore_allimages'] = None

            self.cliargs = {}
            if cliargs:
                # store CLI arguments
                self.cliargs = cliargs

            #update relative to data dir if not an absolute path
            if not os.path.isabs(self.data['image_data_store']):
                self.data['image_data_store'] = os.path.join(self.data['anchore_data_dir'], self.data['image_data_store'])

            if not os.path.exists(self.data['image_data_store']):
                os.makedirs(self.data['image_data_store'])

        except Exception as err:
            self.data = None
            import traceback
            traceback.print_exc()
            raise err

    def __getitem__(self, item):
        """
        Allow dict-style access on the object itself.
        :param item:
        :return:
        """
        return self.data[item]

    def __setitem__(self, key, value):
        """
        Allow dict-style setting of values into the backing structure
        :param key:
        :param value:
        :return:
        """
        self.data[key] = value

    def __str__(self):
        return yaml.safe_dump(self.data)

    def find_config_file(self):
        thefile = ""
        thedir = ""
        if os.path.exists(self.DEFAULT_CONFIG_FILE):
            thefile = self.DEFAULT_CONFIG_FILE
            thedir = self.DEFAULT_CONFIG_DIR
        elif os.path.exists("/etc/anchore/config.yaml"):
            thefile = "/etc/anchore/config.yaml"
            thedir = "/etc/anchore"
        else:
            # config file doesn't exist, copy the example default to ~
            if not os.path.exists(self.DEFAULT_CONFIG_DIR):
                os.makedirs(self.DEFAULT_CONFIG_DIR)
            for d in os.listdir(self.EXAMPLE_CONFIG_DIR):
                if not os.path.exists('/'.join([self.DEFAULT_CONFIG_DIR, d])):
                    shutil.copy('/'.join([self.EXAMPLE_CONFIG_DIR, d]), '/'.join([self.DEFAULT_CONFIG_DIR, d]))

            thefile = self.DEFAULT_CONFIG_FILE
            thedir = self.DEFAULT_CONFIG_DIR

        return thedir, thefile




import sys
import click

from anchore import analyzer, controller, anchore_utils
from anchore.cli.common import build_image_list, anchore_print, anchore_print_err, extended_help_option
from anchore.util import contexts


gate_extended_help = """
Run gate analysis on the specified images with the specified options or else edits the policy that is effective for a specific image or
all images. Anchore gates are a set of checks against the image, image's dockerfile and other data sources (e.g. CVEs). The default gate policy
file is found in $anchore/conf/anchore_gate.policy, where $anchore = $HOME/.anchore (default) or /etc/anchore depending on installation.

Image IDs can be specified as hash ids, repo names (e.g. centos), or tags (e.g. centos:latest).

See the policy file for the set of available gates and their triggers.

\b
The output of each gate is one of:
* GO - Check is ok, no trigger fired
* STOP - Trigger found something. Policy says to output a stop
* WARN - Emit a warning to flag this as something to follow-up on.

The gate results are only output to the user, so their meaning is entirely up to the user. A typical use would be to
not push any container image to production if it has any 'STOP' gate results. But, how to use the gate output is entirely
up to the user.

Editing Gate Policies:

Anchore has a global gate policy document, but this can be overridden for each individual image if desired by
running '--editpolicy' for the desired image. The policy file opened for editing will be specific to the image.
A gate policy file is composed of lines of the following format:

<Gate Name>:<Trigger Name>:<Result if triggered>

Gate Policy Example:

``DOCKERFILECHECK:NOFROM:STOP`` specifies a check against an image's dockerfile (if available) such that if the 'NOFROM' trigger evaluates to true then the gate action will be 'STOP'


Usage Examples:

View gate output for the 'node:latest' image:
'anchore gate --image node:latest'

View gate output for all images:
'anchore gate'

Edit the gate policy for 'nginx:latest':
'anchore gate --image nginx:latest --editpolicy'

"""


@click.command(short_help='Perform and view gate evaluation on selected images')
@click.option('--force', is_flag=True, help='Force gates to run even if they already have.')
@click.option('--image', help='Process specified image ID. Cannot be combined with --imagefile', metavar='<imageid>')
@click.option('--imagefile', help='Process image IDs listed in specified file. Cannot be combined with --image', type=click.Path(exists=True), metavar='<file>')
@click.option('--include-allanchore', help='Include all images known by anchore', is_flag=True)
@click.option('--editpolicy', is_flag=True, help='Edit the gate policies for specified image(s).')
@click.option('--whitelist', is_flag=True, help='Edit evaluated gate triggers and optionally whitelist them.')
@click.pass_obj
@extended_help_option(extended_help=gate_extended_help)
def gate(anchore_config, force, image, imagefile, include_allanchore, editpolicy, whitelist,):
    """
    Runs gate checks on the specified image(s) or edits the image's gate policy.
    The --editpolicy option is only valid for a single image.

    The --image and --imagefile options are mutually exclusive.

    Image IDs can be specified as hash ids, repo names (e.g. centos), or tags (e.g. centos:latest).
    """

    ecode = 0
    success = True

    if image and imagefile:
        raise click.BadOptionUsage('Can only use one of --image, --imagefile')

    try:
        imagedict = build_image_list(anchore_config, image, imagefile, not (image or imagefile), include_allanchore)
        imagelist = imagedict.keys()
    except Exception as err:
        anchore_print_err("could not load any images")
        sys.exit(1)

    try:
        con = controller.Controller(anchore_config=anchore_config, imagelist=imagelist, allimages=contexts['anchore_allimages'], force=force)
    except Exception as err:
        anchore_print_err("gate operation failed")
        ecode = 1
    else:
        if editpolicy:
            if not con.editpolicy():
                ecode = 1
        elif whitelist:
            if not con.editwhitelist():
                ecode = 1
        else:
            try:
                # run the gates
                result = con.run_gates()
                if result:
                    anchore_utils.print_result(anchore_config, result)
                    success = True
                    ecode = con.result_get_highest_action(result)
            except Exception as err:
                print str(err)
                anchore_print_err("failed to run gates")
                ecode = 1

    contexts['anchore_allimages'].clear()
    sys.exit(ecode)


analyze_extended_help = """
Run the analyzer to scan and summarize a container image and store that analysis in the local host's anchore image db for further use in gating, reports, and queries.

Upon completion of analysis of an image there will be an entry in the
db named with the imageid (a directory in $anchore/data/, where
$anchore = $HOME/.anchore (default) or /etc/anchore depending on
installation). If the analyzer detects that an image has already been
analyzed it will skip analysis. This behavior can be overridden with
the '--force' flag.

To get the full power of analysis and subsequent gate and query tools
it is recommended to include the dockerfile as often as possible.
When using --dockerfile, you must use --image and specify a single
image. This allows anchore to associate that specific dockerfile with
the image in the analysis output. Anchore will always try to infer a
dockerfile, but certain constructs and operations in dockerfiles
become opaque after the image is built (e.g. COPY operations, or RUN
operations that invoke a tool that loads data on the image), so
including the actual dockerfile results in the best analysis output
and makes more information available for gate evaluation and later
queries.

Image IDs can be specified as hash ids, repo names (e.g. centos), or tags (e.g. centos:latest).

Image Types: ["none"|"base"]
Specify the type of the image being analyzed. Use 'base' to tell anchore that the given image is a base image to be
used to build other images. Use 'none' to not assign a specific image type to the image.


Examples:

Analyze all images listed in a file that contains lines of the form '<image ID> <path/to/dockerfile>' on the local host:

'anchore analyze --imagefile myimages.txt'

Analyze a newly built image with it's corresponding dockerfile:

'anchore analyze --image myapp:latest --dockerfile myapp.dockerfile'

Analyze all image on the local host except a list (perhaps local tools used in the build process, etc):

'anchore analyze --excludefile exclusions.txt'
"""


@click.command(short_help='Perform analysis on specified image IDs.')
@click.option('--force', is_flag=True, help='Force analysis even if existing analysis is found in db.')
@click.option('--image', help='Process specified image ID', metavar='<imageid>')
@click.option('--imagefile', help='Process image IDs listed in specified file.', type=click.Path(exists=True), metavar='<file>')
@click.option('--include-allanchore', help='Include all images known by anchore, as found in host-local db', is_flag=True)
@click.option('--dockerfile', help='Dockerfile of the image to analyze.', type=click.Path(exists=True), metavar='<file>')
@click.option('--imagetype', help='Specify the type of image that is being analyzed (use "none" if unknown).', metavar='<typetext>')
@click.option('--skipgates', is_flag=True, help='Do not run gates as part of analysis.')
@click.option('--excludefile', help='Name of file containing images to exclude during analysis. Each line is an image name/id', type=click.Path(exists=True), metavar='<file>')
@click.pass_obj
@extended_help_option(extended_help=analyze_extended_help)
def analyze(anchore_config, force, image, imagefile, include_allanchore, dockerfile, imagetype, skipgates, excludefile):
    """
    Invokes the anchore analyzer on the specified image(s).

    To include multiple images use the --imagefile, no option, or --include-allanchore options.
    To exclude specific images from analysis, use the --excludefile option.

    One of --imagetype or --dockerfile are required for an analysis run. Use --dockerfile whenever possible as the inclusion
    of the dockerfile for an image associates the dockerfile and image for later use in queries etc. The --dockerfile option
    is only valid in combination with the --image option.

    When using --imagetype, use 'none' to specify that the image(s) is an unknown or user image and use 'base' to specify
    that the image(s) are approved base images to be used to build other images--'golden' images.

    Image IDs can be specified as hash ids, repo names (e.g. centos), or tags (e.g. centos:latest).

    """

    success = True
    ecode = 0

    args = {}

    if image and imagefile:
        raise click.BadOptionUsage('Can only use one of --image, --imagefile')

    if dockerfile and not image:
        raise click.BadOptionUsage('Must specify --image option when using --dockerfile option')

    if not imagefile:
        if imagetype:
            if imagetype == "anchorebase":
                args['anchorebase'] = True
            elif imagetype == "base":
                args['isbase'] = True
            elif imagetype == "none":
                pass
            else:
                raise click.BadOptionUsage("Invalid imagetype specified: valid types are 'none' or 'base'")
        elif not dockerfile:
            raise click.BadOptionUsage('Must specify either --dockerfile or --imagetype <type>')

    try:
        imagedict = build_image_list(anchore_config, image, imagefile, not (image or imagefile), include_allanchore, exclude_file=excludefile, dockerfile=dockerfile)
        imagelist = imagedict.keys()
    except Exception as err:
        anchore_print_err("could not load any images")
        ecode = 1
    else:

        step = 5
        count = 0
        allimages = {}
        for imageId in imagedict.keys():
            
            if count % step == 0:
                allimages.clear()
                allimages = {}
                count = 0

            args.update({'dockerfile': imagedict[imageId]['dockerfile'], 'skipgates': skipgates})

            inlist = [imageId]
            try:
                anchore_print("Analyzing image: " + imageId)
                success = analyzer.Analyzer(anchore_config=anchore_config, imagelist=inlist, allimages=allimages,force=force, args=args).run()
            except:
                anchore_print_err('failed to run analyzer')
                allimages.clear()
                success = False
                ecode = 1
                break

            count = count + 1
        allimages.clear()

        if not success:
            ecode = 1

    sys.exit(ecode)



import os
import click
import subprocess
import docker
import sys
from anchore.configuration import AnchoreConfiguration
from anchore.version import version as anchore_version

import analyzer
import explorer
import subscriptions
import system
import synchronizer
import logs
import toolbox
import anchore.anchore_image_db

from anchore.util import contexts
from .common import init_output_format, anchore_print_err, extended_help_option


main_extended_help="""
Anchore is a tool for analyzing, querying, and curating container images to deliver the transparency, predictability,
and control necessary to use containers in production. Anchore is composed of a toolset that runs locally on a host as
well as a web service that monitors the container ecosystem and provides inputs to the local toolset.

The tool has capabilities to populate a registry, run analysis, explore/query the analysis results (including custom
queries and checks), and run policy-based gate-functions against container images to help an ops team decide if a container
should go into the CI/CD pipeline and on to production based on attributes of the Dockerfile, built image, or both.

After installation, the first command run must be: 'anchore sync catalog' to initialize the system and load subscription data.

Subscriptions are lists of container images (e.g. ubuntu:latest, centos:7, nginx:latest), that anchore will automatically
fetch from docker hub during a catalog sync to keep up-to-date and will also download the latest analysis data on those images
from the anchore web service. Except during a sync, no network connectivity is required by anchore to run analysis and query
images.

Configuration Files:

Anchore configuration files are automatically installed if not found when looking in-order at:
\b
$HOME/.anchore/conf
/etc/anchore

A default install will copy the files to $HOME/.anchore/conf/, but they may be manually put in /etc/anchore for a system global config.

* config.yaml - main configuration file. Used to override default values. Anchore will search for it in
$HOME/.anchore/conf, then in /etc/anchore. If not found in either place, a new one is initilized and copied to $HOME/.anchore

* anchore_gate.policy - The global gate policy file (see anchore-gate(1) for more help on gates).


High-level example flows:

Initialize the system and subscribe to 'ubuntu' image:

\b
anchore sync catalog
anchore subscriptions add ubuntu
anchore sync catalog

Generate a summary report on all analyzed images

anchore explore report

Check gate output for nginx:latest:

anchore gate --image nginx:latest
"""


@click.group()
@click.option('--verbose', is_flag=True, help='Enable verbose output to stderr.')
@click.option('--debug', is_flag=True, help='Developer debug output to stderr.')
@click.option('--quiet', is_flag=True, help='Only errors to stderr, no status messages.')
@click.option('--json', is_flag=True, help='Output formatted json to stdout.')
@click.option('--plain', is_flag=True, help='Output formatted scriptable text to stdout.')
@click.version_option(version=anchore_version)
@click.pass_context
@extended_help_option(extended_help=main_extended_help)
def main_entry(ctx, verbose, debug, quiet, json, plain):
    """
    Anchore is a tool to analyze, query, and curate container images. The options at this top level
    control stdout and stderr verbosity and format.

    The first command that must be run after installation is: 'anchore sync catalog'. That will initialize
    the system and prepare the local install for use.

    High-level example flows:

    Initialize the system and subscribe to 'ubuntu' image:

    \b
    anchore sync catalog
    anchore subscriptions add ubuntu
    anchore sync catalog

    Generate a summary report on all analyzed images

    anchore explore report

    Check gate output for nginx:latest:

    anchore gate --image nginx:latest


    """
    # Load the config into the context object
    logfile = None
    debug_logfile = None
    try:
        try:
            args = {'verbose': verbose, 'debug': debug, 'json': json, 'plain': plain, 'quiet': quiet}
            anchore_conf = AnchoreConfiguration(cliargs=args)
        except Exception as err:
            click.echo("Error setting up/reading Anchore configuration", err=True)
            click.echo("Info: "+str(err), err=True)
            import traceback
            traceback.print_exc()
            sys.exit(1)
        try:
            logfile = anchore_conf.data['log_file'] if 'log_file' in anchore_conf.data else None
            debug_logfile = anchore_conf.data['debug_log_file'] if 'debug_log_file' in anchore_conf.data else None
        except Exception, e:
            click.echo(str(e))

        ctx.obj = anchore_conf

    except:
        if ctx.invoked_subcommand != 'system':
            click.echo('Expected, but did not find configuration file at %s' % os.path.join(AnchoreConfiguration.DEFAULT_CONFIG_FILE), err=True)
            exit(1)

    try:
        init_output_format(json, plain, debug, verbose, quiet, log_filepath=logfile, debug_log_filepath=debug_logfile)
    except Exception, e:
        click.echo('Error initializing logging: %s' % str(e))
        exit(2)

    if not anchore_pre_flight_check(ctx):
        anchore_print_err("Error running pre-flight checks")
        exit(1)

    if not anchore_common_context_setup(ctx):
        anchore_print_err("Error setting up common data based on configuration")
        exit(1)

main_entry.add_command(subscriptions.subscriptions)
main_entry.add_command(system.system)
main_entry.add_command(synchronizer.sync)
main_entry.add_command(explorer.explore)
main_entry.add_command(analyzer.analyze)
main_entry.add_command(analyzer.gate)
main_entry.add_command(toolbox.toolbox)

def anchore_pre_flight_check(ctx):
    # helper checks
    try:
        subcommand = ctx.invoked_subcommand
        config = ctx.obj.data
    except:
        return(False)
        
    if subcommand in ['explore', 'gate', 'analyze']:

        # check for some shellouts for analyzers
        try:
            from deb_pkg_tools.version import Version
            cmd = ['dpkg-query', '--version']
            sout = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
        except Exception as err:
            anchore_print_err("Anchore requires dpkg libs and commands")
            return(False)

        try:
            from rpmUtils.miscutils import splitFilename
            cmd = ['rpm', '--version']
            sout = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
        except Exception as err:
            anchore_print_err("Anchore requires yum/rpm libs and commands")
            return(False)

        try:
            import graphviz  
            cmd = ['dot', '-V']
            sout = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
        except Exception as err:
            anchore_print_err("Anchore requires graphviz libs and commands")
            return(False)
            
    if subcommand in ['explore', 'gate', 'analyze', 'toolbox']:
        # check DB readiness
        try:
            db = anchore.anchore_image_db.AnchoreImageDB(imagerootdir=config['image_data_store'])
        except Exception as err:
            anchore_print_err("Could not set up connection to Anchore DB")
            return(False)

    return(True)


def anchore_common_context_setup(ctx):
    config = ctx.obj

    if 'docker_cli' not in contexts or not contexts['docker_cli']:
        try:
            contexts['docker_cli'] = docker.Client(base_url=config['docker_conn'])
            testconn = contexts['docker_cli'].images()
        except Exception as err:
            contexts['docker_cli']=None

    if 'anchore_allimages' not in contexts or not contexts['anchore_allimages']:
        contexts['anchore_allimages'] = {}

    if 'anchore_db' not in contexts or not contexts['anchore_db']:
        contexts['anchore_db'] = anchore.anchore_image_db.AnchoreImageDB(imagerootdir=config['image_data_store'])

    return(True)



#!/usr/bin/env python

import sys
import os
import re
import anchore.anchore_utils

try:
    config = anchore.anchore_utils.init_gate_cmdline(sys.argv, "CVE Checking Gate")
except Exception as err:
    print str(err)
    sys.exit(1)

if not config:
    sys.exit(0)

imgid = config['imgid']
imgdir = config['dirs']['imgdir']
analyzerdir = config['dirs']['analyzerdir']
comparedir = config['dirs']['comparedir']
outputdir = config['dirs']['outputdir']

try:
    params = config['params']
except:
    params = None

if not os.path.exists(imgdir):
    sys.exit(0)

metafile = '/'.join([imgdir, 'image_info', 'image.meta'])
meta = {}
FH=open(metafile, 'r')
for l in FH.readlines():
    l=l.strip()
    (k, v) = re.match('(\S*)\s*(.*)', l).group(1, 2)
    meta[k] = v
FH.close()

if meta['usertype'] != "user":
    sys.exit(0)

output = '/'.join([outputdir, 'PKGDIFF'])
OFH=open(output, 'w')

pkgfile = '/'.join([comparedir, 'base', 'package_list', 'pkgs.all'])
if not os.path.exists(pkgfile):
    sys.exit(0)

isdiff = 0
FH=open(pkgfile, 'r')
for l in FH.readlines():
    isdiff = 1
    l = l.strip()

    (pkg, status) = re.match('(\S*)\s*(.*)', l).group(1, 2)

    if (status == 'VERSION_DIFF'):
        OFH.write('PKGVERSIONDIFF Package version in container is different from baseline for pkg - ' + pkg + "\n")
    elif (status == 'INIMG_NOTINBASE'):
        OFH.write("PKGADD Package has been added to image since base - " + pkg + "\n")
    elif (status == 'INBASE_NOTINIMG'):
        OFH.write("PKGDEL Package has been removed to image since base - " + pkg + "\n")
FH.close()

if (isdiff):
    OFH.write("PKGDIFF Package manifest is different from image to base\n")

OFH.close()

sys.exit(0)






#!/usr/bin/env python

import sys
import os
import re
import anchore.anchore_utils

def get_next(image, allimages):
    if image.meta['imageId'] not in allimages:
        allimages[image.meta['imageId']] = image

    imageId = image.meta['imageId']
    i = image.get_latest_userimage()
    if i:
        nimage = anchore.anchore_image.AnchoreImage(i, config['anchore_config']['image_data_store'], allimages)
        return([imageId] + get_next(nimage, allimages))
        
    return([imageId, image.get_earliest_base()])


try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <base image ID> <base image ID> ...\nhelp: use 'all' to show all base image IDs")
except:
    sys.exit(1)

if not config:
    sys.exit(0)

OFH=open(config['output'], 'w')
OFH.write("BaseId Package *ChildImagesWithPackage\n")

allimages = {}
tups = list()
for imageId in config['images']:
    image = anchore.anchore_image.AnchoreImage(imageId, config['anchore_config']['image_data_store'], allimages)
    branch = get_next(image, allimages)
    for i in range(0, len(branch)-1):
        tup = [branch[i], branch[i+1]]
        if not tup in tups:
            tups.append(tup)

pkgout = {}

for t in tups:
    s = t[0]
    d = t[1]
    thedir = '/'.join([config['dirs']['datadir'], s, 'compare_output', d])
    image = allimages[s]
    base = image.get_earliest_base()
    bimage = allimages[d]
    
    if base not in pkgout:
        pkgout[base] = {}

    try:
        report = image.get_compare_report().copy().pop(d, {})
        for l in report['package_list']['pkgs.all']:
            l = l.strip()
            (k, v) = re.match('(\S*)\s*(.*)', l).group(1, 2)
            if not k in pkgout[base]:
                pkgout[base][k] = 0
            pkgout[base][k] = pkgout[base][k] + 1
    except Exception as e:
        pass

for k in pkgout.keys():
    for p in pkgout[k].keys():
        if 'all' in config['params'] or k in config['params'] or k[0:12] in config['params']:
            OFH.write(k[0:12] + " " + p + " " + str(pkgout[k][p]) + "\n")

OFH.close()
allimages.clear()
sys.exit(0)



import graphviz
import os
import logging

from anchore import anchore_utils
from anchore.util import contexts
import anchore_image_db


DEVNULL=open(os.devnull, 'wb')


class Visualizer(object):
    """
    Component that handles generating visualizations of images and trees"
    """

    _logger = logging.getLogger(__name__)

    def __init__(self, config, imagelist, allimages, docker_cli=None):
        self.config = config

        self.allimages = allimages

        self.anchore_datadir = config['image_data_store']
        self.tmproot=self.config['tmpdir']

        self.images = anchore_utils.image_context_add(imagelist,
                                                      allimages,
                                                      docker_cli=docker_cli,
                                                      anchore_datadir=self.anchore_datadir,
                                                      tmproot=self.tmproot,
                                                      anchore_db=contexts['anchore_db'],
                                                      must_be_analyzed=True,
                                                      must_load_all=True)

    def get_images(self):
        return(self.images)

    def get_node_attrs(self, i, mode="family", params={}):
        text = ""
        color = "black"
        style = ""
        fillcolor = ""
        drepo = None
        arepo = None

        arepo = drepo = "n/a"
        if len(i.get_alltags_current()) > 0:
            drepo = ' '.join(i.get_alltags_current())
        if len(i.get_alltags_past()) > 0:
            arepo = ' '.join(i.get_alltags_past())
        
        text = i.meta['shortId'] + "\n(curr="+drepo+")\n(allknown="+arepo+")"
            
        if mode == "family":
            fillcolor = '#FFFFFF'
            if i.is_user():
                fillcolor = '#00AA00'
            elif i.is_anchore_base():
                fillcolor = '#f3d51b'
                #fillcolor = '#AA0000'
            elif i.is_base():
                fillcolor = '#FF0000'
            elif i.is_intermediate():
                fillcolor = '#FFFFFF'

            style = 'filled'

        return([text, color, style, fillcolor, drepo, arepo])

    def vis_graph(self, mode="family", params={}):
        dot = graphviz.Digraph(comment='Container Images: ' + mode)
        nodes = {}
        edges = {}
        show_all = False

        for id in self.images:
            i = self.allimages[id]
            if i.meta['shortId'] not in nodes:

                if show_all or (len(i.get_alltags_ever()) > 0):
                    nodes[i.meta['shortId']] = self.get_node_attrs(i, mode=mode, params=params)

                for fid in i.anchore_familytree:
                    f = self.allimages[fid]

                    if f.meta['shortId'] not in nodes:
                        if show_all or (len(f.get_alltags_ever()) > 0):
                            nodes[f.meta['shortId']] = self.get_node_attrs(f, mode=mode, params=params)

        for id in self.images:
            i = self.allimages[id]
            last = i
            revtree = list(i.get_familytree())
            revtree.reverse()
            for fid in revtree:
                f = self.allimages[fid]

                if show_all or ( len(last.get_alltags_ever()) > 0 and len(f.get_alltags_ever()) > 0 ) :
                    if last.meta['shortId'] != f.meta['shortId']:
                        edges[last.meta['shortId']] = f.meta['shortId']
                        last = f

        for n in nodes.keys():
            dot.node(n, nodes[n][0], color=nodes[n][1], style=nodes[n][2], fillcolor=nodes[n][3])

        for k in edges.keys():
            dot.edge(k, edges[k])

        fname = "/anchore-"+mode+"-graph"
        dot.render('/'.join([self.tmproot,fname]), view=False)
        
    def run(self, params={}):
            self.vis_graph(mode="family")
            self._logger.info("Done: output graph(s) are in: " + '/'.join([self.tmproot,"/anchore-*-graph.pdf"]))




#!/usr/bin/env python

import sys
import os
import re

import anchore.anchore_image
import anchore.anchore_utils

# main routine

try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <image ID to compare to> <another image ID> ...\nhelp: use 'base' to compare against base image")
except:
    sys.exit(1)

if not config:
    sys.exit(0)

outlist = list()
outlist.append(["ImageId", "Repo/Tag", "CompareImageId","Package","InputImageVersion","CompareImageVersion"])

allimages = {}
image = anchore.anchore_image.AnchoreImage(config['imgid'], config['anchore_config']['image_data_store'], allimages)
ipkgs = image.get_allpkgs()
hascontent = False

for fid in config['params']:
    try:
        fimageId = fid

        if fid == 'base':
            fimageId = image.get_earliest_base()

        fimage = anchore.anchore_image.AnchoreImage(fimageId, config['anchore_config']['image_data_store'], allimages)

        fpkgs = fimage.get_allpkgs()

        image_report = image.get_compare_report()

        if fimageId in image_report.keys():

            for p in image_report[fimageId]['package_list']['pkgs.all']:

                (pkg, status) = re.match('(\S*)\s*(.*)', p).group(1, 2)
                ivers = ipkgs.pop(pkg, "NA")

                if status == 'VERSION_DIFF':
                    pvers = fpkgs.pop(pkg, "NA")
                    outlist.append([config['meta']['shortId'], config['meta']['humanname'], fid,pkg,ivers,pvers])
                    hascontent=True
                elif status == 'INIMG_NOTINBASE':
                    outlist.append([config['meta']['shortId'], config['meta']['humanname'], fid,pkg,ivers,"NOTINSTALLED"])
                    hascontent=True
    except:
        pass

if not hascontent:
    #outlist.append(["NOMATCH","NOMATCH","NOMATCH","NOMATCH"])
    pass

anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)

allimages.clear()    
sys.exit(0)



import tarfile
import os
import errno
import hashlib
import io


class ResourceNotFoundException (BaseException):
    def __init__(self, resource, *args, **kwargs):
        super(ResourceNotFoundException, self).__init__(args,kwargs)
        self.message = 'Resource %s not found' % resource


class NoAccessException (BaseException):
    def __init__(self, resource, mode, *args, **kwargs):
        super(NoAccessException, self).__init__(args, kwargs)
        self.message = 'Resource %s does not have correct access mode %s' % (resource, str(mode))


def tarzip_data(root_dir, output_file):
    """
    Given a root directory, adds all its children to a tarball (compressed)
    :param root_dir:
    :param output_file:
    :return:
    """
    if root_dir is None:
        raise ValueError('Must be a valid directory path', root_dir)

    t = tarfile.open(output_file, 'w:gz')
    for child in os.listdir(root_dir):
        t.add(os.path.join(root_dir,child), arcname=child)
    t.close()
    return t.name


def check_path(path, mode=None):
    if path is not None and not os.path.exists(path):
        raise ResourceNotFoundException(path)
    if mode is not None and not os.access(path, mode):
        raise NoAccessException(path, mode)


def createpath(path, mode, exists_ok=True):
    """
    Create directories in the indicated path.
    :param path:
    :param mode:
    :param exists_ok:
    :return:
    """
    try:
        os.makedirs(path, mode)
    except OSError, e:
        if e.errno != errno.EEXIST or not exists_ok:
            raise e


def calc_file_md5(filepath, chunk_size=None):
    """
    Calculate a file's md5 checksum. Use the specified chunk_size for IO or the
    default 256KB

    :param filepath:
    :param chunk_size:
    :return:
    """
    if chunk_size is None:
        chunk_size = 256 * 1024
    md5sum = hashlib.md5()
    with io.open(filepath, 'r+b') as f:
        datachunk = f.read(chunk_size)
        while datachunk is not None and len(datachunk) > 0:
            md5sum.update(datachunk)
            datachunk = f.read(chunk_size)

    return md5sum.hexdigest()




import click

from anchore.catalog import AnchoreCatalog
from anchore.cli.common import anchore_print, anchore_print_err, extended_help_option

working_catalog = None

sync_direction = ['input', 'output', 'both']

sync_extended_help="""
Manage synchronization activities of the local anchore instance with the anchore service. The primary
operation is the catalog sync, which downloads metadata from the anchore web service and pulls the subscribed images
from dockerhub.com.

user-image sync manages the execution of user-supplied scripts to fetch images for local analysis.

There are two basic kinds of syncs: catalog syncs and user-image syncs.

A catalog sync pulls down the latest metadata from the anchore web service as well as any images from Docker Hub as
specified in the current subscriptions list. See anchore-subscriptions(1) for commands to manage the current
set of subscribed containers.

An initial catalog sync is the first command required to initialize the system upon installation. Until a catalog
sync has occured, the local system has no information about what subscriptions are available from the web service.

Requiring explicit sync invocation is by design so that all of the anchore tools can run without connecting to the
Anchore web service once a sync has been completed to enable the administrator to explicitly control when network
access is needed and used.
"""

@click.group(short_help='Synchronize images and metadata')
@click.pass_obj
@extended_help_option(extended_help=sync_extended_help)
def sync(anchore_config):
    """
    Synchronization of images and metadata with the Anchore web service and image sources.

    The first command run on a new installation of anchore must be 'anchore sync catalog' to initialize the local system.
    See the catalog subcommand help for more information.
    """

    # Initialize the registry object
    global working_catalog
    try:
        working_catalog = AnchoreCatalog(config=anchore_config)
    except:
        anchore_print_err('Failed to initialize catalog internal structures. Cannot continue')
        exit(1)


@sync.command(short_help='Show status of local system for synchronizations')
@click.pass_obj
def status(anchore_config):
    """
    Show state of local anchore images and artifacts.

    Returns structure output with the results of checks of local resources and their staleness compared to
    the upstream service artifacts for items such as vulnerability data and analysis db entries for subscription images.

    The output of this command can be used to determine if/when to run a catalog sync and check if new service data is
    available. This command will use the network to check the service status.
    """

    assert anchore_config is not None

    try:
        result = working_catalog.check_status()
        for k,v in result.items():
            if 'sync' in v:
                result[k] = v['sync']

        anchore_print(result, do_formatting=True)
    except:
        anchore_print_err('Failed checking catalog configuration. Please check config file: %s' % anchore_config.config_file)
        exit(1)


@sync.command(name='catalog', short_help='Update the local catalog with new data from the service')
def sync_catalog():
    """
    Updates the local catalog with the latest from the Anchore web service. Pulls CVE data, analysis metadata for images
    subscribed to as well as updates to subscribed images directly from Docker Hub.
    """

    try:
        working_catalog.pull()
    except:
        anchore_print_err('Catalog sync failed')
        exit(1)


@sync.command(name='user-images', short_help='Execute user-provided input/output scripts.')
@click.argument('operation', type=click.Choice(sync_direction))
@extended_help_option()
def user_images(operation):
    """
    Manages fetching and pushing user images. The operations are: 'input', 'output', 'both'.

    By default, 'both' is invoked and will execute 'input' then 'output' operations in that order.

    Scripts are located as follows, assuming $INSTALL_LOC = distro-specific location where pip installs python packages

    input: $INSTALL_LOC/anchore/anchore-modules/inputs/
    output: $INSTALL_LOC/anchore/anchore-modules/outputs/

    Scripts are executed in lexicographic order by filename and scripts must be marked as executable to be run.
    See the README file in each directory for more information.

    """

    if operation == 'input' or operation == 'all':
        try:
            anchore_print('Executing input scripts')
            working_catalog.inputs.execute()
            anchore_print('Execution of input scripts complete')
        except:
            anchore_print_err('Failed executing input scripts')
            exit(1)

    if operation == 'output' or operation == 'all':
        try:
            anchore_print('Executing output scripts')
            working_catalog.outputs.execute()
            anchore_print('Execution of output scripts complete')
        except:
            anchore_print_err('Failed executing output scripts')
            exit(1)




import functools


def trace(f):
    """ Tracing decorator """
    @functools.wraps(f)
    def decorator(*args, **kwargs):
        print 'Calling ' + f.func_name + ' in ' + str(args[0])
        return f(*args, **kwargs)
    return decorator


import os
import re
import subprocess
import random
import shutil
import logging
import time

from anchore import anchore_utils
from anchore.util import contexts


class Navigator(object):
    _logger = logging.getLogger(__name__)

    def __init__(self, anchore_config, imagelist, allimages, docker_cli=None):
        self.config = anchore_config
        self.allimages = allimages
        self.anchore_datadir = self.config['image_data_store']
        self.images = list()

        self.images = anchore_utils.image_context_add(imagelist, allimages, docker_cli=docker_cli, anchore_datadir=self.anchore_datadir, tmproot=self.config['tmpdir'], anchore_db=contexts['anchore_db'], must_be_analyzed=True, must_load_all=True)

    def add_images(self, imagelist):
        newimages = anchore_utils.image_context_add(imagelist, self.allimages, docker_cli=contexts['docker_cli'], anchore_datadir=self.anchore_datadir, tmproot=self.config['tmpdir'], anchore_db=contexts['anchore_db'], must_be_analyzed=True, must_load_all=True)

        self.images = list(set(self.images) | set(newimages))

    def generate_reports(self):
        ret = {}
        for imageId in self.images:
            image = self.allimages[imageId]
            baseId = image.get_earliest_base()
            bimage = self.allimages[baseId]

            image_report = image.get_image_report()
            analysis_report = image.get_analysis_report()
            compare_report = image.get_compare_report()
            gates_report = image.get_gates_report()
            gates_eval_report = image.get_gates_eval_report()

            record = {
                'image_report': image_report,
                'analysis_report': analysis_report,
                'compare_report': compare_report,
                'gates_report': gates_report,
                'gates_eval_report': gates_eval_report,
                # set up printable result
                'result': {
                    'header':['ImageId', 'Type', 'CurrentTags', 'AllTags', 'GateStatus', 'Counts', 'BaseDiffs'],
                    'rows': list()
                }
            }

            shortId = image.meta['shortId']
            usertype = str(image.get_usertype())
            currtags = ','.join(image.get_alltags_current())
            alltags = ','.join(image.get_alltags_ever())

            gateaction = 'UNKNOWN'
            for g in gates_eval_report:
                if g['trigger'] == 'FINAL':
                    gateaction = g['action']
                    break

            pnum = str(len(analysis_report['package_list']['pkgs.all']))
            fnum = str(len(analysis_report['file_list']['files.all']))
            snum = str(len(analysis_report['file_suids']['files.suids']))
            analysis_str = ' '.join(["PKGS="+pnum, "FILES="+fnum, "SUIDFILES="+snum])

            compare_str = "N/A"
            if baseId in compare_report:
                pnum = str(len(compare_report[baseId]['package_list']['pkgs.all']))
                fnum = str(len(compare_report[baseId]['file_list']['files.all']))
                snum = str(len(compare_report[baseId]['file_suids']['files.suids']))
                compare_str = ' '.join(["PKGS="+pnum, "FILES="+fnum, "SUIDFILES="+snum])

            #row = [ fill(x, 80) for x in [ shortId, usertype, currtags, alltags, gateaction, analysis_str, compare_str ] ]
            row = [ shortId, usertype, currtags, alltags, gateaction, analysis_str, compare_str ]

            record['result']['rows'].append(row)

            ret[imageId] = record
        return ret

    def get_images(self):
        return(self.images)

    def get_dockerfile_contents(self):
        result = {}
        for imageId in self.images:
            image = self.allimages[imageId]
            #result[imageId] = image.get_dockerfile_contents()
            record = {'result':{}}
            record['result']['header'] = ['ImageId', 'Mode', 'DockerfileContents']
            record['result']['rows'] = list()
            (dbuf, mode) = image.get_dockerfile_contents()
            record['result']['rows'].append([image.meta['shortId'], mode, dbuf])
            result[imageId] = record
        return(result)

    def get_familytree(self):
        result = {}
        for imageId in self.images:
            image = self.allimages[imageId]
            record = {'result':{}}
            record['result']['header'] = ['ImageId', 'Current Repo/Tags', 'Past Repo/Tags', 'ImageType']
            record['result']['rows'] = list()
            for fid in image.get_familytree():
                fimage = self.allimages[fid]
                fidstr = fimage.meta['shortId']
                curr_tags = ','.join(fimage.get_alltags_current())
                past_tags = ','.join(fimage.get_alltags_past())
                
                usertype = fimage.get_usertype()
                if usertype == "anchorebase":
                    userstr = "Anchore Base Image"
                elif usertype == "oldanchorebase":
                    userstr = "Previous Anchore Base Image"
                elif usertype == "user":
                    userstr = "User Image"
                else:
                    userstr = "Intermediate"

                record['result']['rows'].append([fidstr, curr_tags, past_tags, userstr])

            result[imageId] = record
            
        return(result)

    def get_layers(self):
        result = {}
        for imageId in self.images:
            image = self.allimages[imageId]
            record = {'result':{}}
            record['result']['header'] = ['LayerId']
            record['result']['rows'] = list()
            for fid in image.get_layers() + [imageId]:
                record['result']['rows'].append([fid])

            result[imageId] = record
            
        return(result)

    def get_taghistory(self):
        result = {}
        for imageId in self.images:
            image = self.allimages[imageId]
            record = {'result':{}}
            record['result']['header'] = ['ImageId', 'Date', 'KnownTags']
            record['result']['rows'] = list()
            for tagtup in image.get_tag_history():
                tstr = time.ctime(int(tagtup[0]))
                tagstr = ','.join(tagtup[1])
                record['result']['rows'].append([image.meta['shortId'], tstr, tagstr])

            try:
                currtags = ','.join(image.get_alltags_current())
                record['result']['rows'].append([image.meta['shortId'], time.ctime(), currtags])
            except:
                pass

            result[imageId] = record
        return(result)

    def unpack(self, destdir='/tmp'):
        ret = {}
        for imageId in self.images:
            image = self.allimages[imageId]
            outdir = image.unpack(docleanup=False, destdir=destdir)
            self._logger.debug("Unpacked image " + image.meta['shortId'] + " in " + outdir)
            ret[imageId] = outdir
        return(ret)

    def run(self):
        # navigate images - report, gate, etc
        return(True)

    def find_query_command(self, action):
        querydir = '/'.join([self.config['scripts_dir'], "queries"])
        qcommand = '/'.join([querydir, action])
        check_commands = list()
        check_commands.append(qcommand)
        check_commands.append(qcommand + ".py")
        check_commands.append(qcommand + ".sh")

        qcommand = None
        for q in check_commands:
            if os.path.exists(q):
                qcommand = q
                command_type = 'query'
                break

        if not qcommand:
            mquerydir = '/'.join([self.config['scripts_dir'], "multi-queries"])
            qcommand = '/'.join([mquerydir, action])
            check_commands = list()
            check_commands.append(qcommand)
            check_commands.append(qcommand + ".py")
            check_commands.append(qcommand + ".sh")
            
            qcommand = None
            for q in check_commands:
                if os.path.exists(q):
                    qcommand = q
                    command_type = 'multi-query'
                    break

        if not qcommand:
            return(False, None, None)
        
        return(True, qcommand, command_type)

    def execute_query(self, imglist, qcommand, params):
        success = True
        datadir = self.config['image_data_store']
        outputdir = '/'.join([self.config['anchore_data_dir'], "querytmp", "query." + str(random.randint(0, 99999999))])
        os.makedirs(outputdir)

        imgfile = '/'.join([self.config['anchore_data_dir'], "querytmp", "queryimages." + str(random.randint(0, 99999999))])
        anchore_utils.write_plainfile_fromlist(imgfile, imglist)

        cmd = [qcommand, imgfile, datadir, outputdir] 
        if params:
            cmd = cmd +  params

        meta = {}

        try:
            self._logger.debug("Running query command: " + str(' '.join(cmd)))
            sout = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
            self._logger.debug("Query command execution success")
            if sout:
                self._logger.debug("\tCommand output:\n" + str(sout))
        except subprocess.CalledProcessError as err:
            self._logger.error("Query command ran but execution failed: " )
            self._logger.error("\tCommand: " + ' '.join(cmd))
            self._logger.error("\tException: " + str(err))
            self._logger.error("\tCommand output:\n" + str(err.output))
            self._logger.error("\tExit code: " + str(err.returncode))
            success = False
        except Exception as err:
            self._logger.error("Query command execution call failed: " )
            self._logger.error("\tCommand: " + ' '.join(cmd))
            self._logger.error("\tException: " + str(err))
            success = False
        else:
            try:
                outputs = os.listdir(outputdir)
                if len(outputs) <= 0:
                    raise Exception("No output files found after executing query command\n\tCommand Output:\n"+sout+"\n\tInfo: Query command should have produced an output file in: " + outputdir)

                orows = list()
                ofile = outputs[0]

                try:
                    frows = anchore_utils.read_kvfile_tolist('/'.join([outputdir, ofile]))
                    header = frows[0]
                    rowlen = len(header)
                    for row in frows[1:]:
                        if len(row) != rowlen:
                            raise Exception("Number of columns in data row ("+str(len(row))+") is not equal to number of columns in header ("+str(rowlen)+")\n\tHeader: "+str(header)+"\n\tOffending Row: "+str(row))
                        orows.append(row)
                except Exception as err:
                    raise err 

                meta['queryparams'] = ','.join(params)
                meta['querycommand'] = cmd
                meta['result'] = {}
                meta['result']['header'] = header
                meta['result']['rowcount'] = len(orows)
                try:
                    meta['result']['colcount'] = len(orows[0])
                except:
                    meta['result']['colcount'] = 0
                meta['result']['rows'] = orows

            except Exception as err:
                self._logger.error("Query output handling failed: ")
                self._logger.error("\tCommand: " + ' '.join(cmd))
                self._logger.error("\tException: " + str(err))
                success = False
        finally:
            os.remove(imgfile)

        ret = [success, cmd, outputdir, meta]
        return(ret)

    def list_query_commands(self, command=None):
        result = {}

        record = {'result':{}}
        record['result']['header'] = ["Query", "HelpString"]
        record['result']['rows'] = list()

        querydir = '/'.join([self.config['scripts_dir'], "queries"])
        mquerydir = '/'.join([self.config['scripts_dir'], "multi-queries"])

        for dd in [querydir, mquerydir]:
            for d in os.listdir(dd):
                if re.match(r".*~$|.*#$|.*pyc", d):
                    continue
                d = re.sub(r"\.sh$|\.py$", "", d)
                (rc, qcommand, command_type) = self.find_query_command(d)
                cmd = [qcommand, 'help']
                if not command or command == d:

                    try:
                        self._logger.debug("Running query command: " + str(' '.join(cmd)))
                        sout = subprocess.check_output(cmd, stderr=subprocess.STDOUT)
                        self._logger.debug("Query command execution success")
                        if sout:
                            self._logger.debug("\tCommand output:\n" + str(sout))
                        else:
                            sout = "No usage information for query"
                    except subprocess.CalledProcessError as err:
                        self._logger.error("Query command ran but execution failed: " )
                        self._logger.error("\tCommand: " + ' '.join(cmd))
                        self._logger.error("\tException: " + str(err))
                        self._logger.error("\tCommand output:\n" + str(err.output))
                        self._logger.error("\tExit code: " + str(err.returncode))
                        sout = "Command ran but exited non-zero"
                    except Exception as err:
                        self._logger.error("Query command execution call failed: " )
                        self._logger.error("\tCommand: " + ' '.join(cmd))
                        self._logger.error("\tException: " + str(err))
                        sout = "Command failed to run"

                    record['result']['rows'].append([d, sout])

        result['list_query_commands'] = record
        return(result)

    def run_query(self, query):
        result = {}

        if len(query) == 0:
            return(self.list_query_commands())
        elif len(query) == 1:
            action = query[0]
            params = ['help']
        else:
            action = query[0]
            params = query[1:]

        if re.match(r".*(\.\.|~|/).*", action) or re.match(r"^\..*", action):
            self._logger.error("invalid query string (bad characters in string)")
            return(False)
                    

        (rc, qcommand, command_type) = self.find_query_command(action)
        if not rc:
            self._logger.error("cannot find query command in Anchore query directory: " + action + "(.py|.sh)")
            return(False)

        if params and params[0] == 'help':
            return(self.list_query_commands(action))

        outputdir = None

        if command_type == 'query':
            for imageId in self.images:
                (rc, cmd, outputdir, output) = self.execute_query([imageId], qcommand, params)
                if rc:
                    result[imageId] = output

        elif command_type == 'multi-query':
            (rc, cmd, outputdir, output) = self.execute_query(self.images, qcommand, params)
            if rc:
                result['multi'] = output

        if outputdir:
            shutil.rmtree(outputdir)

        return(result)




import os
import click
import json
import yaml
import logging
import sys
from anchore import anchore_utils
from anchore.cli import logs
from anchore.util import contexts

plain_output = False


def extended_help_option(extended_help=None, *param_decls, **attrs):
    """
    Based on the click.help_option code.

    Adds a ``--extended-help`` option which immediately ends the program
    printing out the extended extended-help page. Defaults to using the
    callback's doc string, but can be given an explicit value as well.

    This is intended for use as a decorator on a command to provide a 3rd level
    of help verbosity suitable for use as a manpage (though not formatted as such explicitly).

    Like :func:`version_option`, this is implemented as eager option that
    prints in the callback and exits.

    All arguments are forwarded to :func:`option`.
    """

    def decorator(f):
        def callback(ctx, param, value):
            if value and not ctx.resilient_parsing:
                if not extended_help:
                    ctx.command.help = ctx.command.callback.__doc__
                    click.echo(ctx.get_help(), color=ctx.color)
                else:
                    ctx.command.help = extended_help
                    click.echo(ctx.get_help(), color=ctx.color)
                ctx.exit()

        attrs.setdefault('is_flag', True)
        attrs.setdefault('expose_value', False)
        attrs.setdefault('help', 'Show extended help content, similar to manpage, and exit.')
        attrs.setdefault('is_eager', True)
        attrs['callback'] = callback
        return click.option(*(param_decls or ('--extended-help',)), **attrs)(f)

    return decorator


def std_formatter(msg):
    """
    Default simple string format. Dumps block-style indented yaml for dicts if found. Otherwise no formatting
    :param msg:
    :return:
    """

    if isinstance(msg, dict):
        return yaml.safe_dump(msg, indent=True, default_flow_style=False)
    return str(msg)


def json_formatter(obj):
    """
    Format the output in JSON
    :param obj:
    :return:
    """
    if isinstance(obj, str):
        # Make a list of size 1
        return json.dumps([obj], indent=True)
    else:
        return json.dumps(obj, indent=True, sort_keys=True)

# Which formatting function to use
formatter = std_formatter


def init_output_format(use_json=False, use_plain=False, use_debug=False, use_verbose=False, use_quiet=False, log_filepath=None, debug_log_filepath = None):
    global formatter

    if use_json:
        formatter = json_formatter

    if use_debug:
        level = 'debug'
    elif use_verbose:
        level = 'verbose'
    elif use_quiet:
        level = 'quiet'
    else:
        level = 'normal'

    logs.init_output_formatters(output_verbosity=level, logfile=log_filepath, debug_logfile=debug_log_filepath)


def anchore_print_err(msg):
    exc = sys.exc_info()
    if exc is not None and exc != (None, None, None):
        logging.getLogger(__name__).exception(msg)
    else:
        logging.getLogger(__name__).error(msg)


def anchore_print(msg, do_formatting=False):
    """
    Print to stdout using the proper formatting for the command.

    :param msg: output to be printed, either an object or a string. Objects will be serialized according to config
    :return:
    """
    if do_formatting:
        click.echo(formatter(msg))
    else:
        click.echo(msg)


def build_image_list(config, image, imagefile, all_local, include_allanchore, dockerfile=None, exclude_file=None):
    """Given option inputs from the cli, construct a list of image ids. Includes all found with no exclusion logic"""

    if not image and not (imagefile or all_local):
        raise click.BadOptionUsage('No input found for image source. One of <image>, <imagefile>, or <all> must be specified')

    if image and imagefile:
        raise click.BadOptionUsage('Only one of <image> and <imagefile> can be specified')

    filter_images = []
    if exclude_file:
        with open(exclude_file) as f:
            for line in f.readlines():
                filter_images.append(line.strip())

    imagelist = {}
    if image:
        imagelist[image] = {'dockerfile':dockerfile}

    if imagefile:
        filelist = anchore_utils.read_kvfile_tolist(imagefile)
        for i in range(len(filelist)):
            l = filelist[i]
            imageId = l[0]
            try:
                dfile = l[1]
            except:
                dfile = None
            imagelist[imageId] = {'dockerfile':dfile}

    if all_local:
        docker_cli = contexts['docker_cli']
        if docker_cli:
            for f in docker_cli.images(all=True, quiet=True, filters={'dangling': False}):
                if f not in imagelist and f not in filter_images:
                    imagelist[f] = {'dockerfile':None}
        else:
            raise Exception("Could not load any images from local docker host - is docker running?")

    if include_allanchore:
        ret = contexts['anchore_db'].load_all_images()
        if ret and len(ret) > 0:
            for l in list(set(imagelist.keys()) | set(ret)):
                imagelist[l] = {'dockerfile':None}

    # Remove excluded items
    for excluded in filter_images:
        docker_cli = contexts['docker_cli']
        if not docker_cli:
            raise Exception("Could not query docker - is docker running?")
        for img in docker_cli.images(name=excluded, quiet=True):
            imagelist.pop(img, None)
            #imagelist.remove(img)

    return imagelist



import copy
import subprocess

import os
import re
import json 
import random 

from anchore import anchore_utils
import logging
import anchore_image_db

from anchore.util import scripting, contexts

DEVNULL=open(os.devnull, 'wb')


class Controller(object):
    """
    Component that manages gate execution on images.
    """

    _logger = logging.getLogger(__name__)

    def __init__(self, anchore_config, imagelist, allimages, force=False):
        self.config = anchore_config
        self.allimages = allimages
        self.force = force        
        self.anchore_datadir = self.config['image_data_store']
        
        if len(imagelist) <= 0:
            raise Exception("No images given to evaluate")

        self.images = anchore_utils.image_context_add(imagelist, allimages, docker_cli=contexts['docker_cli'], anchore_datadir=self.anchore_datadir, tmproot=self.config['tmpdir'], anchore_db=contexts['anchore_db'], must_be_analyzed=True, must_load_all=True)

        self.anchoreDB = contexts['anchore_db']

    def read_policyfile(self, policyfile):
        FH=open(policyfile, 'r')
        policies = {}
        for l in FH.readlines():
            l = l.strip()
            patt = re.compile('^\s*#')

            if (l and not patt.match(l)):
                polinput = l.split(':')
                module = polinput[0]
                check = polinput[1]
                action = polinput[2]
                modparams = ""
                if (len(polinput) > 3):
                    modparams = polinput[3]

                if module not in policies:
                    policies[module] = {}

                if check not in policies[module]:
                    policies[module][check] = {}

                policies[module][check]['action'] = action
                policies[module][check]['params'] = modparams

        FH.close()
        return(policies)

    def merge_policies(self, poldst, polsrc):
        polret = copy.deepcopy(poldst)
        for sk in polsrc.keys():
            if sk not in polret:
                polret[sk] = polsrc[sk]

            for smk in polsrc[sk].keys():
                if smk not in polret[sk]:
                    polret[sk] = polsrc[sk]
        
        return(polret)

    def save_policyfile(self, policy, outpolicyfile):
        FH=open(outpolicyfile, 'w')
        for k in policy.keys():
            for c in policy[k].keys():
                if policy[k][c]['params']:
                    outline = k + ":" + c + ":" + policy[k][c]['action'] + ":" + policy[k][c]['params']
                else:
                    outline = k + ":" + c + ":" + policy[k][c]['action']
                FH.writelines(outline + "\n")
        FH.close()
        return(True)

    def get_images(self):
        return(self.images)

    def get_image_policies(self, image):
        # load default and image override policies, merge (if new
        # checks are in default), and save (if there is a diff after
        # the merge)

        default_gatepol = '/'.join([self.config.config_dir, "anchore_gate.policy"])
        image_gatepol = image.anchore_imagedir + "/anchore_gate.policy"

        default_policies = self.read_policyfile(default_gatepol)
        image_policies = False

        if os.path.exists(image_gatepol):
            image_policies = self.read_policyfile(image_gatepol)

        if image_policies and default_policies:
            policies = self.merge_policies(image_policies, default_policies)
            if policies != image_policies:
                self.save_policyfile(policies, image_gatepol)
        else:
            policies = default_policies
            self.save_policyfile(policies, image_gatepol)

        return(policies)

    def load_whitelist(self, image):
        ret = {'ignore':[], 'enforce':[]}

        whitelist = '/'.join([image.get_imagedir(), "/anchore_gate.whitelist"])
        if not os.path.exists(whitelist):
            return(ret)

        FH=open(whitelist, 'r')
        for l in FH.readlines():
            l = l.strip()
            try:
                if re.match("^#.*", l):
                    l = re.sub("^#", "", l)
                    json_dict = json.loads(l)
                    ret['ignore'].append(json_dict)
                else:
                    json_dict = json.loads(l)
                    ret['enforce'].append(json_dict)
            except:
                pass
        FH.close()        
        return(ret)

    def save_whitelist(self, image, loaded, latest):
        new = {'ignore':list(loaded['ignore']), 'enforce':loaded['enforce']}

        whitelist = '/'.join([image.get_imagedir(), "/anchore_gate.whitelist"])
        if not os.path.exists(whitelist):
            FH=open(whitelist, 'w')
            for i in latest:
                FH.write(json.dumps(i) + "\n\n")
            FH.close()
        else:
            newpol = False
            for i in latest:
                # add evaled policy if not in the loaded whitelist
                if i not in new['ignore'] and i not in new['enforce']:
                    new['enforce'].append(i)
                    newpol = True

            # write the new whitelist, adding any new policies
            if newpol:
                FH=open(whitelist, 'w')
                for i in new['ignore']:
                    FH.write("#"+json.dumps(i) + "\n\n")
                for i in new['enforce']:
                    FH.write(json.dumps(i) + "\n\n")
                FH.close()
        return(True)

    def evaluate_gates_results(self, image):
        ret = list()
        final_gate_action = 'GO'

        policies_whitelist = self.load_whitelist(image)

        policies = self.get_image_policies(image)
        for m in policies.keys():
            opath = image.anchore_imagedir + "/gates_output/" + m
            if os.path.exists(opath):
                FH=open(opath, 'r')
                for l in FH.readlines():
                    l = l.strip()
                    (k, v) = re.match('(\S*)\s*(.*)', l).group(1, 2)
                    if k in policies[m]:
                        r = {'imageId':image.meta['imageId'], 'check':m, 'trigger':k, 'output':v, 'action':policies[m][k]['action']}
                        # this is where whitelist check should go
                        if r not in policies_whitelist['ignore']:
                            if policies[m][k]['action'] == 'STOP':
                                final_gate_action = 'STOP'
                            elif final_gate_action != 'STOP' and policies[m][k]['action'] == 'WARN':
                                final_gate_action = 'WARN'
                            ret.append(r)
                        else:
                            # whitelisted, skip evaluation
                            pass
                FH.close()
        
        self.save_whitelist(image, policies_whitelist, ret)

        ret.append({'imageId':image.meta['imageId'], 'check':'FINAL', 'trigger':'FINAL', 'output':"", 'action':final_gate_action})
        
        for i in ret:
            if os.path.exists(image.get_imagedir() + "/gates_output/" + i['check'] + ".eval"):
                os.remove(image.get_imagedir() + "/gates_output/" + i['check'] + ".eval")

        for i in ret:
            FH=open(image.get_imagedir() + "/gates_output/" + i['check'] + ".eval", 'a')
            FH.write(i['trigger'] + " " + i['action'] + "\n")
            FH.close()

        self.anchoreDB.save_gates_eval_report(image.meta['imageId'], ret)
        return(ret)

    def execute_gates(self, image, refresh=True):
        self._logger.debug("gate policy evaluation for image "+str(image.meta['imagename'])+": begin")
        success = True

        imagename = image.meta['imageId']
        imagedir = image.anchore_imagedir
        gatesdir = '/'.join([self.config["scripts_dir"], "gates"])
        outputdir = imagedir + "/gates_output"
        workingdir = '/'.join([self.config['anchore_data_dir'], 'querytmp'])
        
        if not self.force and os.path.exists(imagedir + "/gates.done"):
            self._logger.info(image.meta['shortId'] + ": evaluated.")
            return(True)

        self._logger.info(image.meta['shortId'] + ": evaluating policies ...")
        
        if not os.path.exists(imagedir):
            os.makedirs(imagedir)

        if not os.path.exists(outputdir):
            os.makedirs(outputdir)

        if not os.path.exists(workingdir):
            os.makedirs(workingdir)

        imgfile = '/'.join([workingdir, "queryimages." + str(random.randint(0, 99999999))])
        anchore_utils.write_plainfile_fromstr(imgfile, image.meta['imageId'])

        policies = self.get_image_policies(image)
        paramlist = list()
        for p in policies.keys():
            for t in policies[p].keys():
                if 'params' in policies[p][t] and policies[p][t]['params']:
                    paramlist.append(policies[p][t]['params'])
        if len(paramlist) <= 0:
            paramlist.append('all')

        results = scripting.ScriptSetExecutor(path=gatesdir).execute(capture_output=True, fail_fast=True, cmdline=' '.join([imgfile, self.config['image_data_store'], outputdir, ' '.join(paramlist)]))

        os.remove(imgfile)

        for r in results:
            (cmd, retcode, output) = r
            if retcode:
                self._logger.error("FAILED")
                self._logger.error("\tCMD: " + cmd)
                self._logger.error("\tEXITCODE: " + str(retcode))
                self._logger.error("\tOUTPUT: " + output)
                success = False
            else:
                self._logger.debug("")
                self._logger.debug("\tCMD: " + cmd)
                self._logger.debug("\tEXITCODE: " + str(retcode))
                self._logger.debug("\tOUTPUT: " + output)
                self._logger.debug("")

        if success:
            report = self.generate_gates_report(image)
            self.anchoreDB.save_gates_report(image.meta['imageId'], report)
            self._logger.info(image.meta['shortId'] + ": evaluated.")

        self._logger.debug("gate policy evaluation for image "+str(image.meta['imagename'])+": end")
        return(success)

    def generate_gates_report(self, image):
        # this routine reads the results of image gates and generates a formatted report
        report = {}

        analysisdir = image.anchore_imagedir + "/gates_output/"
        for d in os.listdir(analysisdir):
            if re.match(".*\.eval$", d):
                continue

            if d not in report:
                report[d] = list()

            report[d] = anchore_utils.read_plainfile_tolist('/'.join([analysisdir, d]))
            #FH = open(analysisdir + "/" + d, 'r')
            #for l in FH.readlines():
            #    l = l.strip()
            #    report[d].append(l)
            #FH.close()
        return(report)

    def result_get_highest_action(self, results):
        highest_action = 0
        for k in results.keys():
            action = results[k]['result']['final_action']
            if action == 'STOP':
                highest_action = 1
            elif highest_action == 0 and action == 'WARN':
                highest_action = 2
            
        return(highest_action)

    def run_gates(self, refresh=True):
        # actually run the gates
        ret = {}
        for imageId in self.images:
            image = self.allimages[imageId]

            if not self.execute_gates(image, refresh=refresh):
                raise Exception("One or more gates failed to execute")

            results = self.evaluate_gates_results(image)

            record = {}
            record['result'] = {}
            record['result']['header'] = ['ImageID', 'Repo/Tag', 'Gate', 'Trigger', 'CheckOutput', 'GateAction']
            record['result']['rows'] = list()
            report = results
            for m in report:
                id = image.meta['imageId']
                name = image.get_human_name()
                gate = m['check']
                trigger = m['trigger']
                output = m['output']
                action = m['action']
                row = [id[0:12], name, gate, trigger, output, action]
                record['result']['rows'].append(row)
                if gate == 'FINAL':
                    record['result']['final_action'] = action

            ret[imageId] = record
        return(ret)

    def editpolicy(self):
        return(self.edit_policy_file(editpolicy=True))

    def editwhitelist(self):
        return(self.edit_policy_file(whitelist=True))

    def edit_policy_file(self, editpolicy=False, whitelist=False):
        ret = True

        if editpolicy:
            polfile = "anchore_gate.policy"
        elif whitelist:
            polfile = "anchore_gate.whitelist"
        else:
            # nothing to do
            return(ret)

        for imageId in self.images:
            image = self.allimages[imageId]            
            policies = self.get_image_policies(image)        

            thefile = '/'.join([image.anchore_imagedir, polfile])
                
            if not os.path.exists(thefile):
                self._logger.info("Cannot find file to edit, skipping: " + str(thefile))
            else:
                if "EDITOR" in os.environ:
                    cmd = os.environ["EDITOR"].split()
                    cmd.append(thefile)
                    try:
                        subprocess.check_output(cmd, shell=False)
                    except:
                        ret = False
                elif os.path.exists("/bin/vi"):
                    try:
                        rc = os.system("/bin/vi " + thefile)
                        if rc:
                            ret = False
                    except:
                        ret = False
                else:
                    self._logger.info("Cannot find editor to use: please set the EDITOR environment variable and try again")
                    break
                    ret = False

        return(ret)




import os
import logging

import anchore_image_db
import controller
from anchore import anchore_utils
from anchore.util import scripting
from anchore.util import contexts


class Analyzer(object):
    _logger = logging.getLogger(__name__)

    def __init__(self, anchore_config, imagelist, allimages, force, args=None, docker_cli=None):
        self._logger.debug("analyzer initialization: begin")

        self.config = anchore_config
        self.allimages = allimages
        self.force = force
        self.anchore_datadir = self.config['image_data_store']

        self.dockerfile = None
        try:
            self.dockerfile = args['dockerfile']
        except:
            pass

        self.skipgates = False
        try:
            self.skipgates = args['skipgates']
        except:
            pass

        try:
            if 'isbase' in args and args['isbase']:
                usertype = 'base'
            elif 'anchorebase' in args and args['anchorebase']:
                usertype = 'anchorebase'
            else:
                usertype = None
        except:
            usertype = None

        self._logger.debug("init input processed, loading input images: " + str(imagelist))
        
        self.images = anchore_utils.image_context_add(imagelist, allimages, docker_cli=docker_cli, dockerfile=self.dockerfile, anchore_datadir=self.anchore_datadir, tmproot=self.config['tmpdir'], anchore_db=contexts['anchore_db'], usertype=usertype, must_load_all=True)

        self._logger.debug("loaded input images, checking that all input images have been loaded " + str(self.images))

        self.anchoreDB = contexts['anchore_db']

        self._logger.debug("analyzer initialization: end")

    def get_images(self):
        return(self.images)

    def run_analyzers(self, image):
        self._logger.debug("running analyzers on image: " + str(image.meta['imagename']) + ": begin")

        imagename = image.meta['imagename']
        outputdir = image.anchore_imagedir
        shortid = image.meta['shortId']
        analyzerdir = '/'.join([self.config["scripts_dir"], "analyzers"])

        if not self.force and os.path.exists(outputdir + "/analyzers.done"):
            self._logger.debug("image already analyzed and --force was not specified, nothing to do")
            self._logger.info(image.meta['shortId'] + ": analyzed.")

            return (True)

        self._logger.info(image.meta['shortId'] + ": analyzing ...")

        if not os.path.exists(outputdir):
            self._logger.debug("outputdir '" + str(outputdir) + "'not found, creating")
            os.makedirs(outputdir)

        self._logger.debug("unpacking image")
        imagedir = image.unpack()
        self._logger.debug("finished unpacking image to directory: " + str(imagedir))

        self._logger.debug("running all analyzers on image")
        results = scripting.ScriptSetExecutor(path=analyzerdir).execute(capture_output=True, fail_fast=True,
                                                                        cmdline=' '.join(
                                                                            [imagename, self.config['image_data_store'], outputdir, imagedir]))
        self._logger.debug("analyzers done running" + str(len(results)))

        success = True
        for r in results:
            (cmd, retcode, output) = r
            if retcode:
                # something failed
                self._logger.error("FAILED")
                self._logger.error("\tCMD: " + cmd)
                self._logger.error("\tEXITCODE: " + str(retcode))
                self._logger.error("\tOUTPUT: " + output)
                success = False
            else:
                self._logger.debug("")
                self._logger.debug("\tCMD: " + cmd)
                self._logger.debug("\tEXITCODE: " + str(retcode))
                self._logger.debug("\tOUTPUT: " + output)
                self._logger.debug("")

        self._logger.debug("analyzer commands all finished with successful exit codes")
        if success:
            self._logger.debug("generating analysis report from analyzer outputs and saving")
            report = self.generate_analysis_report(image)
            self.anchoreDB.save_analysis_report(image.meta['imageId'], report)

            self._logger.debug("saving image information with updated analysis data")
            image.save_image()

            anchore_utils.touch_file(outputdir + "/analyzers.done")

            self._logger.info(image.meta['shortId'] + ": analyzed.")

        self._logger.debug("running analyzers on image: " + str(image.meta['imagename']) + ": end")
        return (success)

    def generate_analysis_report(self, image):
        # this routine reads the results of image analysis and generates a formatted report
        report = {}

        analysisdir = image.anchore_imagedir + "/analyzer_output/"
        for d in os.listdir(analysisdir):
            if d not in report:
                report[d] = {}

            moduledir = analysisdir + "/" + d
            for o in os.listdir(moduledir):
                datafile = moduledir + "/" + o
                if o not in report[d]:
                    report[d][o] = list()
                report[d][o] = anchore_utils.read_plainfile_tolist(datafile)
        return (report)

    def generate_compare_report(self, image):
        # this routine reads the results of image compare and generates a formatted report
        report = {}

        rootdir = image.anchore_imagedir + "/compare_output/"
        for b in os.listdir(rootdir):
            if b not in report:
                report[b] = {}

            comparedir = rootdir + "/" + b + "/"
            for d in os.listdir(comparedir):
                if d == 'differs.done':
                    continue

                if d not in report[b]:
                    report[b][d] = {}

                moduledir = comparedir + "/" + d
                for o in os.listdir(moduledir):
                    datafile = moduledir + "/" + o
                    
                    if o not in report[b][d]:
                        report[b][d][o] = list()
                    report[b][d][o] = anchore_utils.read_plainfile_tolist(datafile)

        return (report)

    def run_differs(self, image, baseimage):
        self._logger.debug("comparison of " + str(image.meta['imagename']) + " to " + str(image.meta['imagename']) + ": begin")
        shortida = image.meta['shortId']
        shortidb = baseimage.meta['shortId']

        if not image.is_analyzed():
            self._logger.error("cannot compare image " + shortida + " - need to analyze first.")
            return (False)

        if not baseimage.is_analyzed():
            self._logger.error("cannot compare image " + shortidb + " - need to analyze first")
            return (False)

        outputdir = image.anchore_imagedir

        if not self.force and os.path.exists(
                                        outputdir + "/compare_output/" + baseimage.meta['imageId'] + "/differs.done"):
            self._logger.debug("images already compared and --force not specified, nothing to do")
            self._logger.info(shortida + " to " + shortidb + ": compared.")
            return (True)

        self._logger.info(shortida + " to " + shortidb + ": comparing ...")

        if not os.path.exists(outputdir):
            self._logger.debug("output directory '" + str(outputdir) + "' does not exist, creating")
            os.makedirs(outputdir)

        thedir = outputdir + "/compare_output/" + baseimage.meta['imageId'] + "/"
        if not os.path.exists(thedir):
            self._logger.debug("output directory '" + str(thedir) + "' does not exist, creating")
            os.makedirs(thedir)

        self._logger.debug("getting analysis reports for images")
        areport = image.get_analysis_report()
        breport = baseimage.get_analysis_report()

        self._logger.debug("performing comparison")
        for azkey in areport.keys():
            if azkey in breport:
                for aokey in areport[azkey].keys():
                    if aokey in breport[azkey]:
                        outputdict = {}

                        adatadict = {}
                        for l in areport[azkey][aokey]:
                            l = l.strip()
                            (k, v) = l.split()
                            adatadict[k] = v

                        bdatadict = {}
                        for l in breport[azkey][aokey]:
                            l = l.strip()
                            (k, v) = l.split()
                            bdatadict[k] = v

                        for dkey in adatadict.keys():
                            if not dkey in bdatadict:
                                outputdict[dkey] = "INIMG_NOTINBASE"
                            elif adatadict[dkey] != bdatadict[dkey]:
                                outputdict[dkey] = "VERSION_DIFF"

                        for dkey in bdatadict.keys():
                            if not dkey in adatadict:
                                outputdict[dkey] = "INBASE_NOTINIMG"

                        thedir = outputdir + "/compare_output/" + baseimage.meta['imageId'] + "/" + azkey + "/"
                        if not os.path.exists(thedir):
                            os.makedirs(thedir)

                        
                        thefile = thedir + "/" + aokey
                        anchore_utils.write_kvfile_fromdict(thefile, outputdict)

        self._logger.debug("all comparisons completed")

        anchore_utils.touch_file(outputdir + "/compare_output/" + baseimage.meta['imageId'] + "/differs.done")

        self._logger.info(shortida + " to " + shortidb + ": compared.")

        self._logger.debug("comparison of " + str(image.meta['imagename']) + " to " + str(image.meta['imagename']) + ": end")
        return (True)

    def run(self):
        self._logger.debug("main image analysis on images: " + str(self.images) + ": begin")
        # analyze image and all of its family members
        success = True
        toanalyze = {}
        comparehash = {}
        linkhash = {}

        # calculate all images to be analyzed
        for imageId in self.images:
            coreimage = self.allimages[imageId]

            toanalyze[coreimage.meta['imageId']] = coreimage

            base = False
            lastimage = coreimage
            for i in coreimage.anchore_familytree:
                image = self.allimages[i]
                toanalyze[image.meta['imageId']] = image

                if (image.meta['shortId'] != coreimage.meta['shortId'] and not image.is_intermediate()):
                    comparehash[coreimage.meta['shortId'] + image.meta['shortId']] = [coreimage, image]
                    comparehash[lastimage.meta['shortId'] + image.meta['shortId']] = [lastimage, image]
                    if not base and image.is_base():
                        base = image
                    lastimage = image

            if base:
                linkhash[image.meta['imageId']] = base.meta['imageId']

        # execute analyzers
        self._logger.debug("images to be analyzed: " + str(toanalyze.keys()))
        for imageId in toanalyze.keys():
            image = toanalyze[imageId]
            success = self.run_analyzers(image)
            if not success:
                self._logger.error("analyzer failed to run on image " + str(image.meta['imagename']) + ", skipping the rest")
                break

        if not success:
            self._logger.error("analyzers failed to run on one or more images.")
            return (False)

        # execute differs
        for k in comparehash.keys():
            a = comparehash[k][0]
            b = comparehash[k][1]
            if a.meta['imageId'] != b.meta['imageId']:
                self.run_differs(a, b)

            self._logger.debug("generating and saving comparison report")
            report = self.generate_compare_report(a)
            self.anchoreDB.save_compare_report(a.meta['imageId'], report)

        for k in linkhash.keys():
            image = self.allimages[k]
            base = self.allimages[linkhash[k]]
            if image.meta['imageId'] != base.meta['imageId']:
                self._logger.debug("found image (" + image.meta['imageId'] + ") base (" + base.meta[
                    'imageId'] + "), creating softlink 'base'")
                dpath = image.anchore_imagedir + "/compare_output/base"
                if os.path.exists(dpath):
                    os.remove(dpath)
                os.symlink(image.anchore_imagedir + "/compare_output/" + base.meta['imageId'], dpath)

        if not self.skipgates:
            # execute gates
            self._logger.debug("running gates post-analysis: begin")
            for imageId in toanalyze.keys():
                c = controller.Controller(anchore_config=self.config, imagelist=[imageId], allimages=self.allimages).run_gates(refresh=True)
            self._logger.debug("running gates post-analysis: end")

        self._logger.debug("main image analysis on images: " + str(self.images) + ": end")
        return (success)



#!/usr/bin/env python

import sys
import os
import shutil
import re
import json
import time
import subprocess
import anchore.anchore_utils

analyzer_name = "analyzer_meta"

try:
    config = anchore.anchore_utils.init_analyzer_cmdline(sys.argv, analyzer_name)
except Exception as err:
    print str(err)
    sys.exit(1)

imgname = config['imgid']
outputdir = config['dirs']['outputdir']
unpackdir = config['dirs']['unpackdir']

# figure out the distro/version
meta = {}
outfile = outputdir + "/analyzer_meta"
osfile = unpackdir + "/rootfs/etc/os-release"
if os.path.exists(osfile):
    FH=open(osfile, 'r')
    for l in FH.readlines():
        l = l.strip()
        try:
            (key, val) = l.split("=")
            val = re.sub(r'"', '', val)
            if key == "ID":
                meta['DISTRO'] = val
            elif key == "VERSION_ID":
                meta['DISTROVERS'] = val
            elif key == "ID_LIKE":
                meta['LIKEDISTRO'] = ','.join(val.split())
        except:
            a=1

    FH.close()
elif os.path.exists(unpackdir + "/rootfs/etc/system-release-cpe"):
    FH=open(unpackdir +"/rootfs/etc/system-release-cpe", 'r')
    for l in FH.readlines():
        l = l.strip()
        try:
            distro = l.split(':')[2]
            vers = l.split(':')[4]
            meta['DISTRO'] = distro
            meta['DISTROVERS'] = vers
        except:
            pass
    FH.close()
elif os.path.exists(unpackdir + "/rootfs/bin/busybox"):
    meta['DISTRO'] = "busybox"
    try:
        sout = subprocess.check_output([unpackdir+"/rootfs//bin/busybox"])
        fline = sout.splitlines(True)[0]
        slist = fline.split()
        meta['DISTROVERS'] = slist[1]
    except:
        meta['DISTROVERS'] = "0"

if 'DISTRO' not in meta:
    meta['DISTRO'] = "Unknown"
    meta['DISTROVERS'] = "0"


FH=open(outfile, 'w')
for k in meta.keys():
    FH.writelines(k + " " + meta[k] + "\n")
FH.close()
shutil.copy(outfile, unpackdir + "/analyzer_meta")

sys.exit(0)





#!/usr/bin/env python

import sys
import os
import re
import anchore.anchore_utils

def perform_cve_scan(cvedatadir='/tmp', severity='all'):
    severity = severity
    ret = {}
    cve_ret = {}

    for imageId in config['images']:
        image = allimages[imageId]
        cve_data = anchore.anchore_utils.cve_load_data(cvedatadir, image)

        earliest_vulnerable = None

        if cve_data:
            cve_results = anchore.anchore_utils.cve_scanimage(cve_data, image)

            if cve_results:
                for cveId in cve_results.keys():
                    if severity == 'all' or cve_results[cveId]['severity'] == severity:
                        depth=0
                        branch = list()
                        for fid in image.get_familytree():
                            fimage = allimages[fid]
                            if fimage.is_user() or fimage.is_base() or fid == image.meta['imageId']:
                                if fid == image.meta['imageId']:
                                    fcve_results = cve_results.copy()
                                else:
                                    fcve_results = anchore.anchore_utils.cve_scanimage(cve_data, fimage)

                                if cveId in fcve_results.keys():
                                    branch.append(fid)
                                    cve_ret = cve_ret.copy()
                                    cve_ret.update(fcve_results)

                        if len(branch) > 0:
                            if cveId not in ret:
                                ret[cveId] = list()
                            ret[cveId].append(branch)

    branches = ret
    cve_results = cve_ret

    ret = {}
    for cveId in branches.keys():

        tofix = {}
        torebuild = {}

        for b in branches[cveId]:
            image = allimages[b[0]]

            tofix[''.join(image.get_human_string().split())] = True
            for bk in b[1:]:
                image = allimages[bk]
                torebuild[''.join(image.get_human_string().split())] = True

            total_vuln = len(tofix.keys()) + len(torebuild.keys())

            if not cveId in ret:
                ret[cveId] = list()

            if len(torebuild.keys()) <= 0:
                rebuildstr = 'None'
            else:
                rebuildstr = ','.join(torebuild.keys())

            if len(tofix.keys()) <= 0:
                fixstr = 'None'
            else:
                fixstr = ','.join(tofix.keys())

        if cve_results[cveId]['fixVers'] == 'None':
            fixpkg = "None"
        else:
            fixpkg = cve_results[cveId]['pkgName'] + "-" + cve_results[cveId]['fixVers']
        ret[cveId].append([cveId, cve_results[cveId]['severity'], str(total_vuln), cve_results[cveId]['pkgName'] + "-" + cve_results[cveId]['imageVers'], fixpkg, fixstr, rebuildstr ])
    return(ret)

# main routine

# parse cmdline into config dict
try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <Low|Medium|High|Critical|Unknown|Negligible> <another level> ...\nhelp: use 'all' to scan for all severity levels")
except Exception as err:
    print str(err)
    sys.exit(1)

# this means that no config is available, for example when 'help' is given as a parameter
if not config:
    sys.exit(0)

# handle default case if no params are given
if len(config['params']) <= 0:
    config['params'] = ['all']

# set up output list
outlist = list()
outlist.append(["CVEID","Severity","*TotalAffected","VulnerablePackage", "FixAvailable","FixImages","RebuildImages"])

# perform query
allimages = {}
for imageId in config['images']:
    image = anchore.anchore_image.AnchoreImage(imageId, config['anchore_config']['image_data_store'], allimages)

    if imageId not in allimages:
        allimages[imageId] = image
        
for sev in config['params']:
    try:
        cvedatadir = '/'.join([config['anchore_config']['image_data_store'], '..', 'cve-data'])
        result = perform_cve_scan(cvedatadir, sev)

        for k in result.keys():
            for kk in result[k]:
                outlist.append(kk)
    except:
        pass

# save output list as output file for anchore to ingress
anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)

# cleanup and exit
allimages.clear()
sys.exit(0)



#!/usr/bin/env python

import sys
import os
import shutil
import re
import json
import time
import rpm
import subprocess

import anchore.anchore_utils

analyzer_name = "package_list"

try:
    config = anchore.anchore_utils.init_analyzer_cmdline(sys.argv, analyzer_name)
except Exception as err:
    print str(err)
    sys.exit(1)

imgname = config['imgid']
outputdir = config['dirs']['outputdir']
unpackdir = config['dirs']['unpackdir']

#imgname = sys.argv[1]
#datadir = sys.argv[2]
#imgdatadir = sys.argv[3]
#unpackdir = sys.argv[4]
#outputdir = imgdatadir + "/analyzer_output/" + analyzer_name

if not os.path.exists(outputdir):
    os.makedirs(outputdir)

metafile = unpackdir + "/analyzer_meta"
meta = {"DISTRO":"Unknown", "DISTROVERS":"0", "LIKEDISTRO":"Unknown"}
meta.update(anchore.anchore_utils.read_kvfile_todict(metafile))

if meta['DISTRO'] in ['centos', 'rhel'] or meta['LIKEDISTRO'] in ['centos', 'rhel']:
    flav = "RHEL"
elif meta['DISTRO'] in ['ubuntu', 'debian'] or meta['LIKEDISTRO'] in ['ubuntu','debian']:
    flav = "DEB"
elif meta['DISTRO'] in ['busybox'] or meta['LIKEDISTRO'] in ['busybox']:
    flav = 'BUSYB'
else:
    flav = "UNK"

FH=open(outputdir + "/pkgs.all", 'w')
CFH=open(outputdir + "/pkgs_plus_source.all", 'w')

if flav == "RHEL":
    try:
        rpm.addMacro("_dbpath", unpackdir + "/rootfs/var/lib/rpm")
        ts = rpm.TransactionSet()
        mi = ts.dbMatch()
        for h in mi:
            FH.write(h['name'] + " " + h['version'] + "-" + h['release'] + "\n")
    except:
        try:
            sout = subprocess.check_output(['chroot', unpackdir + '/rootfs', 'rpm', '--queryformat', '%{NAME} %{VERSION}-%{RELEASE}\n', '-qa'])
            for l in sout.splitlines():
                l = l.strip()
                FH.write(l + "\n")
        except:
            pass
elif flav == "DEB":
    actual_packages = {}
    all_packages = {}
    other_packages = {}
    cmd = ["dpkg-query", "--admindir="+unpackdir+"/rootfs/var/lib/dpkg", "-W", "-f="+"${Package} ${Version} ${source:Package} ${source:Version}\\n"]
    try:
        sout = subprocess.check_output(cmd)
        for l in sout.splitlines(True):
            l = l.strip()
            (p, v, sp, sv) = re.match('(\S*)\s*(\S*)\s*(\S*)\s*(.*)', l).group(1, 2, 3, 4)
            if p and v:
                if p not in actual_packages:
                    actual_packages[p] = v
                if p not in all_packages:
                    all_packages[p] = v
            if sp and sv:
                if sp not in all_packages:
                    all_packages[sp] = sv
            if p and v and sp and sv:
                if p == sp and v != sv:
                    other_packages[p] = [sv]

    except Exception as err:
        print "Could not run command: " + str(cmd)
        print "Exception: " + str(err)
        print "Please ensure the command 'dpkg' is available and try again"
        raise err

    for p in actual_packages.keys():
        FH.write(' '.join([p, actual_packages[p], '\n']))
        
    for p in all_packages.keys():
        CFH.write(' '.join([p, all_packages[p], '\n']))

    if len(other_packages) > 0:
        for p in other_packages.keys():
            for v in other_packages[p]:
                CFH.write(' '.join([p, v, '\n']))

elif flav == "BUSYB":
    FH.write("BusyBox " + meta['DISTROVERS'] + "\n")
else:
    FH.write("Unknown 0\n")
FH.close()
CFH.close()



import sys
import click

from anchore.cli.common import build_image_list, anchore_print, anchore_print_err, extended_help_option
from anchore import visualizer, navigator, anchore_image_db, anchore_utils
from anchore.util import contexts

config = {}
imagelist = []
nav = None
vis = None

@click.group(short_help='Search, report and query specified image IDs.')
@click.option('--image', help='Process specified image ID', metavar='<imageid>')
@click.option('--imagefile', help='Process image IDs listed in specified file', type=click.Path(exists=True), metavar='<file>')
@click.option('--include-allanchore', help='Include all images known by anchore', is_flag=True)
@click.pass_obj
@extended_help_option()
def explore(anchore_config, image, imagefile, include_allanchore):
    """
    Explore image content via queries, visualizations and reports for the selected image(s).

    Image IDs can be specified as hash ids, repo names (e.g. centos), or tags (e.g. centos:latest).

    """
    global config, imagelist, nav, vis
    ecode = 0
    success = True
    config = anchore_config

    if image and imagefile:
        raise click.BadOptionUsage('Can only use one of --image, --imagefile')

    try:
        imagedict = build_image_list(anchore_config, image, imagefile, not (image or imagefile), include_allanchore)
        imagelist = imagedict.keys()
    except Exception as err:
        anchore_print_err("could not load any images")
        sys.exit(1)

def init_nav_vis_contexts():
    try:
        # use the obj from the current click context. This is a bit hacky, but works as long as this method is
        # invoked in an execution context of click
        anchore_config = click.get_current_context().obj
        nav = navigator.Navigator(anchore_config=anchore_config, imagelist=imagelist, allimages=contexts['anchore_allimages'])
        vis = visualizer.Visualizer(config=anchore_config, imagelist=imagelist, allimages=contexts['anchore_allimages'])
        return nav, vis
    except Exception as err:
        anchore_print_err("explore operation failed")
        success = False
        ecode = 1

    if not success:
        contexts['anchore_allimages'].clear()
        sys.exit(ecode)

@explore.command(short_help='Generate relationship graph of images.')
@extended_help_option()
def visualize():
    """
    Visualization provides a graphical representation of the relationship between images.

    Output is a set of image files in the tmp dir specified in the anchore config.yaml file or /tmp by default.

    """
    ecode = 0
    args={}
    try:
        nav, vis = init_nav_vis_contexts()
        vis.run()
    except:
        anchore_print_err("visualize operation failed")
        ecode = 1

    contexts['anchore_allimages'].clear()

    sys.exit(ecode)


@explore.command(short_help='Run specified query (leave blank to show list).')
@click.argument('module', nargs=-1, metavar='<modulename>')
@extended_help_option()
def query(module):
    """
    Execute the specified query (module) with any parameters it requires. Modules are scripts in a specific location.

    Each query has its own parameters and outputs.

    Examples using pre-defined queries:

    Query all images to see which have the package 'wget' installed:
    'anchore explore query has-package wget'

    """
    ecode = 0
    try:
        nav, vis = init_nav_vis_contexts()

        result = nav.run_query(list(module))
        if result:
            anchore_utils.print_result(config, result)

    except:
        anchore_print_err("query operation failed")
        ecode = 1

    contexts['anchore_allimages'].clear()
    sys.exit(ecode)


@explore.command(short_help='Show analysis report of the specified image(s).')
@extended_help_option()
def report():
    """
    Show analysis report of the specified image(s).

    The analysis report includes information on:

    \b
    Image Id - The image id (as a hash)

    Type - The type of image (--imagetype option used when anchore analyze was run)

    CurrentTags - The current set of repo tags on the image

    AllTags - The set of all repo tags that have been on the image during analysis passes

    GateStatus - The overall aggregate gate output status: GO|STOP|WARN

    Counts - The counts for various attributes of the images such as packages, files, and suid files

    BaseDiffs - Differences of this image from its base image

    Report outputs these entries in a table format by default.
    """
    ecode = 0

    try:
        nav, vis = init_nav_vis_contexts()

        result = nav.generate_reports()
        if result:
            anchore_utils.print_result(config, result)

    except:
        anchore_print_err("operation failed")

        ecode = 1

    contexts['anchore_allimages'].clear()
    sys.exit(ecode)





#!/usr/bin/env python

import sys
import os
import re
import anchore.anchore_utils

# main routine

try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <STOP|WARN|GO> ...\nhelp: use 'all' to show all gateactions")
except:
    sys.exit(1)

if not config:
    sys.exit(0)

if 'all' in config['params']:
    config['params'] = ['STOP', 'WARN', 'GO']

OFH=open(config['output'], 'w')
OFH.write("ImageID Repo/Tag Gate Trigger Action\n")

try:
    gfiles = os.listdir(config['dirs']['gatesdir'])
    for g in gfiles:
        gatefile = '/'.join([config['dirs']['gatesdir'], g])
        patt = re.match("(.*)\.eval$", g)
        if not patt:
            continue

        gate = patt.group(1)
        triggers = {}
        FH=open(gatefile, 'r')
        for l in FH.readlines():
            l = l.strip()
            (trigger, action) = re.match('(\S*)\s*(.*)', l).group(1, 2)

            if trigger != 'FINAL':
                continue

            if action in config['params'] and trigger not in triggers:
                OFH.write(config['meta']['shortId'] + " " + config['meta']['humanname'] + " " + gate + " " + trigger + " " + action + "\n")
                triggers[trigger] = action
                hascontent=True
        FH.close()
    if not hascontent:
        pass
except:
    pass

OFH.close()
sys.exit(0)



#!/usr/bin/env python

import sys
import os
import re

import anchore.anchore_image
import anchore.anchore_utils

# main routine

try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <image ID to compare to> <another image ID> ...\nhelp: use 'base' for comparison against base image")
except:
    sys.exit(1)

if not config:
    sys.exit(0)

if len(config['params']) <= 0:
    config['params'] = ['base']

outlist = list()
outlist.append(["ImageID", "Repo/Tag", "CompareID", "File", "InputImageFileChecksum","CompareImageChecksum"])

allimages = {}
image = anchore.anchore_image.AnchoreImage(config['imgid'], config['anchore_config']['image_data_store'], allimages)
ipkgs = image.get_allfiles()

hascontent = False

for fid in config['params']:
    try:
        fimageId = fid

        if fid == 'base':
            fimageId = image.get_earliest_base()

        fimage = anchore.anchore_image.AnchoreImage(fimageId, config['anchore_config']['image_data_store'], allimages)
        fpkgs = fimage.get_allfiles()

        image_report = image.get_compare_report()
        if fimageId in image_report.keys():
            for p in image_report[fimageId]['file_checksums']['files.md5sums']:
                (pkg, status) = re.match('(\S*)\s*(.*)', p).group(1, 2)
                ivers = ipkgs.pop(pkg, "NA")
                if status == 'VERSION_DIFF':
                    pvers = fpkgs.pop(pkg, "NA")
                    outlist.append([config['meta']['shortId'], config['meta']['humanname'], fid, pkg, ivers, pvers])
                    hascontent=True
                elif status == 'INIMG_NOTINBASE':
                    outlist.append([config['meta']['shortId'], config['meta']['humanname'], fid, pkg, ivers, "NOTINSTALLED"])
                    hascontent=True
    except Exception as err:
        print err
        pass

if not hascontent:
    pass


anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)

allimages.clear()    
sys.exit(0)



import datetime
import json
import logging
import tarfile
import docker
import os

from anchore.configuration import AnchoreConfiguration
from anchore.util import fs_util, scripting
from anchore.util.resources import ResourceCache

module_logger = logging.getLogger(__name__)


class Checker(object):
    """
    Generic checker for validating resources
    """
    PASS = 'pass'
    FAIL = 'fail'

    def __init__(self, resource):
        self._resource_to_check = resource

    def __call__(self):
        return self.check()

    def check(self):
        """
        Child classes must implement this function
        :return: dict with at least one 'status': '[PASS|FAIL]' item
        """
        raise NotImplementedError


class RepoSubscription(object):
    """
    The set of containers that the local registry will get updates for from upstream.
    """
    _logger = logging.getLogger(__name__)

    def __init__(self, path):
        self._file_path = path

        if not os.path.exists(path):
            self._logger.debug('No existing subscription data file found. Initializing new empty subscription')
            self._repos = []
            self.flush()
        else:
            self._logger.debug('Loading subscription data from backing file: %s' % self._file_path)
            with open(self._file_path) as f:
                self._repos = json.load(f)

    def flush(self):
        self._logger.debug('Flushing subscription data to backing file: %s' % self._file_path)
        try:
            with open(self._file_path, 'w') as fd:
                json.dump(self._repos, fd)
        except:
            self._logger.error('Failed to flush subscription list to file: %s' % self._file_path)
            raise

    def get(self):
        return self._repos

    def update(self, repos):
        """
        Adds repo entries to the subscription. If no tag is specified, then 'latest' is assumed and added
        :param repos:
        :return:
        """
        self._logger.debug('Adding %s to subscription list' % str(repos))
        for r in repos:
            if r not in self._repos:
                self._repos.append(r)

    def remove(self, repos):
        self._logger.debug('Removing %s from subscription list' % str(repos))
        for r in repos:
            if r in self._repos:
                self._repos.remove(r)


class AnalysisMetadataManager(object):
    """
    Syncs metadata (analysis output) from the hosted service. Requires the AWS CLI installed and operational
    to do the rsync-like operations
    """
    _logger = logging.getLogger(__name__)

    engine_file = 'engine.json'
    image_list_filename = 'analysis_mapping.json'

    def __init__(self, remote_sync_url, sync_dir, resource_cache, image_list_filename=None, engine_filename=None):
        self._logger.debug('Initializing analysis metadata sync manager')
        assert remote_sync_url is not None
        assert sync_dir is not None
        assert resource_cache is not None

        if engine_filename is not None:
            self.engine_file = engine_filename

        if image_list_filename is not None:
            self.image_list_file = os.path.join(self.sync_dir, image_list_filename)

        self.sync_dir = sync_dir
        self.remote_sync_url = remote_sync_url

        self._images_processed = self._load_processed_map()
        self.engine_repos, self.engine_tags = self._load_engine_repos()
        self.resource_cache = resource_cache

    def _load_processed_map(self):
        try:
            self._logger.debug('Loading list of processed images from %s' % self.image_list_file)
            if os.path.exists(self.image_list_file):
                with open(self.image_list_file) as fd:
                    return json.load(fd)
            else:
                return []
        except:
            self._logger.debug('No existing image list found. Using empty')
            return []

    def _load_engine_repos(self):
        engine_file = os.path.join(self.sync_dir, 'engine.json')
        try:
            if not os.path.exists(engine_file):
                return [], []

            self._logger.debug('Loading engine repos from %s' % engine_file)
            with open(engine_file) as source_list:
                output = json.load(source_list)
                return output['repos'], output['tags']
        except IOError:
            # Nothing local.
            self._logger.warn('Could not open, load, or initialize engine repos list')
            self._logger.debug('Failed to load engine repos list', exc_info=1)
            return None, None

    def has_synced(self):
        """
        Return true if there is analysis data present from the service
        :return:
        """
        # represent it by the engine repos/tags. If either is present, sync was done at some point
        return len(self.engine_repos) > 0 or len(self.engine_tags) > 0 and os.path.exists(self.image_list_file)

    def initialize(self):
        """
        Idempotently initialize the metadata system by checking local resources and setting up local resources.
        Does *NOT* perform a metadata sync.
        """
        try:
            if not os.path.exists(self.sync_dir):
                self._logger.debug('Initializing and creating/checking data dir: %s' % (self.sync_dir))
                fs_util.createpath(self.sync_dir, mode=0755, exists_ok=True)
        except Exception:
            self._logger.exception('Failed initializing analysis metadata directory')
            raise

    def fetch(self):
        """
        Pull the latest analysis metadata from the upstream service subject to local subscriptions

        :return:
        """

        # Safely call since idempotent
        self.initialize()

        self._logger.info('Syncing analysis metadata from Anchore service')
        old_processed = self._images_processed
        result = self.resource_cache.get(self.remote_sync_url)
        if self.remote_sync_url.endswith('.tar.gz'):
            tarfile.open(result['content']).extractall(path=self.sync_dir)

        self._images_processed = self._load_processed_map()

    def check(self):
        """
        Check the sync state of the local system relative to the upstream service. Indicate if changes are pending.
        :return:
        """

        result = {}
        # Check sync values
        result['sync'] = {}
        try:
            if self.resource_cache.is_stale(self.remote_sync_url):
                result['sync']['state'] = 'out-of-date'
            else:
                result['sync']['state'] = 'up-to-date'
            result['sync']['local_cache_metadata'] = self.resource_cache.lookup(self.remote_sync_url)
        except KeyError:
            result['sync']['state'] = 'not-synced'
        except:
            self._logger.exception('Error checking staleness of analysis db data.')
            result['sync']['state'] = 'unknown'

        result['local'] = {}
        result['local']['data_dir'] = self.sync_dir
        result['local']['data_dir_found'] = os.path.exists(self.sync_dir)
        result['local']['db_initialized'] = self.has_synced()

        return result

class ImageManager(object):
    """
    Manages images and syncing images with the local docker repo
    """
    _logger = logging.getLogger(__name__)
    _default_tag = 'latest'

    def __init__(self, docker_url):
        assert docker_url is not None
        self._docker_url = docker_url

    def initialize(self):
        return True

    def fetch(self, image_list):
        """
        Pull the images from docker
        :return:
        """

        client = docker.Client(base_url=self._docker_url)
        for img in image_list:
            self._logger.info('Pulling image: %s' % str(img))
            if ':' in img and '@' not in img:
                img_comp = img.split(':')
                # Pull specific tag
                self._logger.debug('Syncing repo: ' + img + ' with tag = ' + self._default_tag)
                self._logger.debug(client.pull(repository=img_comp[0], tag=img_comp[1]))
            elif '@' in img:
                # It's a hash. Pass verbatim
                self._logger.debug('Syncing hash-identified image: ' + img)
                self._logger.debug(client.pull(repository=img))
            else:
                # It's repo, assume latest...
                # can later add full-repo support by removing the 'tag' here
                self._logger.debug('Syncing assumed "latest" tag for repo: ' + img)
                self._logger.debug(client.pull(repository=img, tag='latest'))

    def check(self):
        try:
            with docker.Client(base_url=self._docker_url) as client:
                info = client.info()
        except:
            return {'status': 'fail', 'docker_url': self._docker_url, 'detail': 'could not get info from docker at url'}

        return {'status': 'pass' if info is not None else 'fail', 'docker_url':self._docker_url }


class VulnerabilityManager(object):
    _logger = logging.getLogger(__name__)

    _output_dir = 'cve-data'

    def __init__(self, data_dir, remote_sync_url, resource_cache):
        assert remote_sync_url is not None
        assert resource_cache is not None and isinstance(resource_cache, ResourceCache)

        self._output_dir = os.path.join(data_dir, self._output_dir)

        self._remote_sync_url = remote_sync_url
        self.resource_cache = resource_cache

        self.initialize()

    def initialize(self):
        fs_util.createpath(self._output_dir, mode=0755, exists_ok=True)

    def has_data(self):
        """
        True if data exists in the output dir as proxy for a sync has been done
        :return:
        """
        return len(os.listdir(self._output_dir)) > 0

    def check(self):
        self._logger.debug('Checking vulnerabilities data upstream')
        result = {}

        # Check sync values
        result['sync'] = {}
        try:
            if self.resource_cache.is_stale(self._remote_sync_url):
                result['sync']['state'] = 'out-of-date'
            else:
                result['sync']['state'] = 'up-to-date'
            result['sync']['local_cache_metadata'] = self.resource_cache.lookup(self._remote_sync_url)
        except KeyError:
            result['sync']['state'] = 'not-synced'
        except:
            self._logger.exception('Error checking staleness of vulnerability data.')
            result['sync']['state'] = 'unknown'

        # check local
        result['local'] = {}
        result['local']['resource_cache_location'] = self.resource_cache.path
        result['local']['output_dir'] = self._output_dir
        result['local']['output_dir_found'] = os.path.exists(self._output_dir)
        result['local']['has_data'] = self.has_data()

        return result

    def fetch(self):
        try:
            self.initialize()  # safe idempotent op
            self._logger.info('Syncing vulnerability data from Anchore service')
            response_obj = self.resource_cache.get(self._remote_sync_url)

            if response_obj is not None:
                # Untar
                self._logger.debug('Extracting tarball contents to into %s' % self._output_dir)
                try:
                    with tarfile.open(response_obj['content']) as tarball:
                        tarball.extractall(path=self._output_dir)
                except:
                    self._logger.error('Failed to extract tarball contents.')
                    raise

        except Exception, e:
            self._logger.error('Failed to download vulnerability data: %s' % e.message)
            raise e


class UserScriptsHandler:
    def __init__(self, path):
        self.script_executor = scripting.ScriptSetExecutor(path=path)

    def execute(self):
        return self.script_executor.execute(capture_output=True, fail_fast=False)


class AnchoreCatalog(object):
    """
    Main anchore manager for the local host
    """

    _logger = logging.getLogger(__name__)
    _config_section = 'manager'
    _input_module_path = 'inputs'
    _output_module_path = 'outputs'
    _subscription_filename = 'subscription.json'
    _resource_cache_name = 'resource_cache'

    def __init__(self, config=None):
        if config is not None:
            if not isinstance(config, AnchoreConfiguration):
                raise TypeError('config parameter must be a RegistryConfiguration instance')
            else:
                self._config = config
                self._load()
        else:
            self._config = None
            self.metadata = None
            self.vulnerabilities = None
            self.images = None
            self.subscription = None

        self.inputs = UserScriptsHandler(path='/'.join([self._config['scripts_dir'], self._input_module_path]))
        self.outputs = UserScriptsHandler(path='/'.join([self._config['scripts_dir'], self._output_module_path]))

    def _load(self):
        resource_cache_dir = os.path.join(self._config['anchore_data_dir'], self._resource_cache_name)
        self.resource_cache = ResourceCache(path=resource_cache_dir,
                                            progress_bars=('quiet' not in self._config.cliargs or not self._config.cliargs['quiet']))

        self.metadata = AnalysisMetadataManager(remote_sync_url=self._config['analysis']['url'],
                                                sync_dir=self._config['image_data_store'],
                                                resource_cache=self.resource_cache)

        self.vulnerabilities = VulnerabilityManager(data_dir=self._config['anchore_data_dir'],
                                                    remote_sync_url=self._config['vulnerabilities']['url'],
                                                    resource_cache=self.resource_cache)

        self.images = ImageManager(docker_url=self._config['images']['docker_conn'])

        self.subscription = RepoSubscription(path=os.path.join(self._config['anchore_data_dir'], self._subscription_filename))

    def has_db(self):
        """
        Has the system been initialized and the sync run at least once.
        :return: true if sync has been run (however old)
        """

        # The analysis data is the critical one to check
        return self.metadata.has_synced()

    def initialize(self, configuration=None):
        if self._config is None and (configuration is None or not isinstance(configuration, AnchoreConfiguration)):
            raise ValueError('configuration cannot be None or non-RegistryConfiguration'
                             ' if this instance does not already have a config')

        if self._config is None:
            self._config = configuration
            self._load()

        self._logger.info('Creating new anchore registry')
        self._logger.info('Initializing Metadata')
        self.metadata.initialize()
        self._logger.info('Initializing Images')
        self.images.initialize()
        self._logger.info('Initializing Vulnerabilities')
        self.vulnerabilities.initialize()

    def clean(self):
        # delete_git_repo(path=repo_path(registry_home))
        # delete_docker_registry(DEFAULT_REPO_CONTAINER_NAME)
        self._logger.info('Cleaning registry.')

    def configuration(self):
        return self._config

    def check_status(self):
        """
        Returns a dict with status output for each dependency

        :return:
        """
        analysis_status = self.metadata.check()
        image_status = self.images.check()
        vulnerabilities_status = self.vulnerabilities.check()
        # docker_registry_status = check_docker_registry(registry_home)
        status = {'Analysis Metadata': analysis_status, 'Vulnerabilities Data': vulnerabilities_status,
                  'Images': image_status}
        return status

    def pull(self):
        """
        Synchronizes the local vulnerability db with the live version upstream
        :return:
        """

        self.metadata.fetch()

        self.vulnerabilities.fetch()

        self.images.fetch(self.subscription.get())

        return True

    def subscribe(self, repo_list):
        """
        Add the given repos/tags to the subscription list after validation checks.
        If repos/tags are not available for subscription then a ValueError is raised
        :param repo_list:
        :return:
        """

        tagged_list = self._add_implicit_tags(repo_list)
        invalid = self._filter_unavailable(tagged_list)
        if invalid is None or len(invalid) == 0:
            self.subscription.update(repos=tagged_list)
            self.subscription.flush()
        else:
            raise ValueError('Containers not available to subscribe to: %s' % str(invalid))

    @staticmethod
    def _add_implicit_tags(img_list, default='latest'):
        return [ x + ':' + default if ':' not in x and '@' not in x else x for x in img_list ]


    def unsubscribe(self, repo_list):
        tagged_list = self._add_implicit_tags(repo_list)
        self.subscription.remove(repos=tagged_list)
        self.subscription.flush()

    def _filter_unavailable(self, repos):
        """
        Filter the given list and return a new list of repos that are not valid. Return [] if none
        :param containers:
        :return:
        """
        self._logger.debug('Checking engine images list source for valid subscription values: %s'
                           % str(self.metadata.engine_tags))
        tag_set = set(self.metadata.engine_tags)
        candidate_set = set(repos)

        return list(candidate_set.difference(tag_set))

    @staticmethod
    def backup(anchore_config, destdir='/tmp'):
        if not isinstance(anchore_config, AnchoreConfiguration):
            raise TypeError('anchore_config must be an instance of AnchoreConfiguration')

        dateval = datetime.datetime.now().isoformat('-')
        backupfile = os.path.join(destdir, 'anchore-backup-{0}.tar.gz'.format(dateval))

        data_dir = anchore_config['anchore_data_dir']
        image_dir = anchore_config['image_data_store']

        module_logger.info('Backing up anchore to: %s' % backupfile)
        # Just tar up the whole thing
        with tarfile.TarFile.open(name=backupfile, mode='w:gz') as tf:
            tf.add(data_dir)
            if not data_dir in image_dir:
                # It's outside the regular data dir tree, so explicitly include
                tf.add(image_dir)
            if not data_dir in anchore_config.config_dir:
                tf.add(anchore_config.config_dir)

        return backupfile

    @staticmethod
    def restore(dest_root, backup_file):
        """
        Restore saved anchore backup
        :param dest_root: string path to root dir to restore. Will install files there relative to original install dirs
        :param backup_file: a file obj or string-path to the backup file
        :return:
        """
        if not os.path.exists(dest_root):
            raise StandardError('Destination root dir does not exist')

        if isinstance(backup_file, str) and not os.path.exists(backup_file):
            raise StandardError('Backup file %s not found' % backup_file)

        if isinstance(backup_file, str):
            module_logger.info('Restoring anchore from file %s to path: %s' % (backup_file, dest_root))
            with tarfile.TarFile.open(backup_file, mode='r:gz') as tf:
                tf.extractall(path=dest_root)

        else:
            module_logger.info('Restoring anchore from file %s to path: %s' % (backup_file.name, dest_root))
            with tarfile.TarFile.open(fileobj=backup_file, mode='r:gz') as tf:
                tf.extractall(path=dest_root)

        return dest_root


# Main entry point for the registry manager cli
import click
import yaml

import anchore.catalog
from anchore.catalog import AnchoreCatalog
from anchore.cli.common import anchore_print, anchore_print_err


@click.group()
def system():
    """
    Anchore system-level operations, which do not operate on resources, but the overall system itself.

    System includes subcommands to backup, restore, and view status info on the local system.
    """
    pass


@system.command()
@click.option('--config', is_flag=True, help='Output the currently used configuration yaml content')
@click.pass_obj
def status(anchore_config, config):
    """
    Print state of local anchore images and artifacts. Includes paths, cache states, and configuration values.

    Use the --config option to dump the current configuration that the system is using. The configuration option
    returns structured output (yaml) or json if using the --json option. All configuration values are populated,
    with defaults if not explicitly overridden in the config file. The output of this is suitable to create a new config
    file.


    """
    assert anchore_config is not None

    try:

        working_catalog = AnchoreCatalog(config=anchore_config)
        if config:
            if anchore_config.cliargs['json']:
                anchore_print(working_catalog.configuration().data, do_formatting=True)
            else:
                anchore_print(yaml.safe_dump(working_catalog.configuration().data, indent=True, default_flow_style=False))
        else:
            result = working_catalog.check_status()
            for k, v in result.items():
                if 'local' in v:
                    result[k] = v['local']

            anchore_print(result, do_formatting=True)

    except:
        anchore_print_err('Failed checking local system status. Please check config file: %s' % anchore_config.config_file)
        exit(1)


@system.command()
@click.argument('outputdir', type=click.Path())
@click.pass_obj
def backup(anchore_config, outputdir):
    """
    Backup the Anchore data locally to a tarball. Will result in a backup file with the name:
    anchore-backup-<date>.tar.gz

    If the anchore configuration file specifies a different image_data_store outside of the anchore_data_dir tree it will
    be backed up, but may require manual intervention on restore. Backup includes the configuration files as well as data
    files. Backup does *not* include docker images themselves.

    """

    try:
        output_file = anchore.catalog.AnchoreCatalog.backup(anchore_config, outputdir)
        anchore_print({'output': output_file}, do_formatting=True)
    except:
        anchore_print_err('Backup of catalog to %s failed' % outputdir)
        exit(1)


@system.command(short_help='Restore Anchore data from a tarball')
@click.argument('inputfile', type=click.File('rb'))
@click.argument('destination_root', type=click.Path(), default='/')
@click.pass_obj
def restore(anchore_config, inputfile, destination_root):
    """
    Restore anchore from a backup to directory root. E.g. "anchore system restore /tmp/anchore_backup.tar.gz /"

    If the image_data_store value has been changed from default ('data') to a path outside of the anchore_data_dir subtree
    then manual intervention may be required to modify the restored config file or move the image_data_store if relative paths
    were used that are no longer accessible. The restore process is an untar process so data is placed in the same place
    relative to the root directory when the tar was created. If the system being restored on is structured differently than
    the original backup source then manual movement of data or config file values may be necessary to get all artifacts in the
    correct locations for the system to find.

    """

    anchore_print('Restoring anchore registry from backup file %s to %s'
               % (inputfile, destination_root))
    try:
        anchore.catalog.AnchoreCatalog.restore(destination_root, inputfile)
    except:
        anchore_print_err('Restore of catalog from %s failed' % inputfile)
        exit(1)




#!/usr/bin/python
from setuptools import setup, find_packages
from anchore import version

import os, shutil, errno

installroot = '/'
linux_default_config_location = os.path.join(installroot, 'etc/anchore')


def install_configs(overwrite=False):
    if overwrite:
        shutil.copytree('/usr/etc/anchore', linux_default_config_location)
    else:
        prefix = '/usr/etc/anchore'
        if not os.path.isdir(linux_default_config_location):
            try:
                os.makedirs(linux_default_config_location)
            except OSError:
                if errno != 17:
                    raise

        for f in os.listdir(prefix):
            oldfile = os.path.join(prefix, f)
            newfile = os.path.join(linux_default_config_location, f)

            if os.path.exists(newfile):
                shutil.copyfile(newfile, newfile + '.old')
                shutil.copyfile(oldfile, newfile)


with open('requirements.txt') as f:
    requirements = f.read().splitlines()

package_name = "anchore"

package_data = {
    package_name: ['conf/*',
                   'anchore-modules/analyzers/*',
                   'anchore-modules/gates/*',
                   'anchore-modules/queries/*',
                   'anchore-modules/multi-queries/*',
                   'anchore-modules/inputs/*',
                   'anchore-modules/outputs/*',
                   'anchore-modules/shell-utils/*',
                   'anchore-modules/examples/queries/*',
                   'anchore-modules/examples/multi-queries/*',
                   'anchore-modules/examples/analyzers/*',
                   'anchore-modules/examples/gates/*',
                   'doc/man/*'
                   ]
}

scripts = ['scripts/anchore_bash_completer']

anchore_description = 'A toolset for inspecting, querying, and curating containers'
anchore_long_description = open('README.rst').read()

url = 'https://github.com/anchore/anchore.git'

data_files = []

setup(
    name='anchore',
    author='Anchore Inc.',
    author_email='dev@anchore.com',
    license='Apache License 2.0',
    description=anchore_description,
    long_description=anchore_long_description,
    url=url,
    packages=find_packages(exclude=('conf*', 'tests*')),
    version=version,
    data_files=data_files,
    include_package_data=True,
    package_data=package_data,
    entry_points='''
    [console_scripts]
    anchore=anchore.cli:main_entry
    ''',
    install_requires=requirements,
    scripts=scripts
)



#!/usr/bin/env python

import sys
import os
import re
import anchore.anchore_utils

# parse the commandline and set up local config
try:
    config = anchore.anchore_utils.init_query_cmdline(sys.argv, "params: <package> <package> ...\nhelp: search input image(s) for specified <package> installations")
except Exception as err:
    print str(err)
    sys.exit(1)
if not config:
    sys.exit(0)

# at this point, the 'config' dict contains (minimally)
# config['name'] : name for this module
# config['imgid'] : image ID of image to be queried
# config['dirs']['datadir'] : top level directory of the anchore image data store
# config['dirs']['imgdir'] : location of the files that contain useful image information
# config['dirs']['analyzerdir'] : location of the files that contain results of the image analysis
# config['dirs']['comparedir'] : location of the files that contain comparison results between image and other images in its familytree
# config['dirs']['gatesdir'] : location of the results of the latest anchore gate evaulation
# config['params'] : any extra parameters passed in from the CLI to this module

# set up the output row array
outlist = list()

# the first value in the output array must contain column header names (no whitespace!)
outlist.append(["FullImageID", "FileName", "IsFound"])

# perform your check

# read the file that container a list of all files in the container that is a result of previous anchore analysis
allfiles = '/'.join([config['dirs']['analyzerdir'], 'file_list', 'files.all'])
    
# use helpful anchore util to read the contents of analyzer file into a dict
files = anchore.anchore_utils.read_kvfile_todict(allfiles)

# perform your check
if "./etc/passwd" in files.keys():
    outlist.append([config['imgid'], "/etc/passwd", "Yes"])
else:
    outlist.append([config['imgid'], "/etc/passwd", "No"])

# use helpful anchore util to write the resulting list to the correct location
anchore.anchore_utils.write_kvfile_fromlist(config['output'], outlist)

# all done!
sys.exit(0)

# To try this query:
# 1) place this example script in the queries directory (cp query-example.py $ANCHOREROOT/lib/anchore/queries/query-example)
# 2) ensure that the script is executable (chmod +x $ANCHOREROOT/lib/anchore/queries/query-example)
# 3) run the anchore query against an analyzed container (anchore explore --image centos query query-example all)
